@@ -46,23 +46,34 @@ sub go_to {
          return;
       }
    }
-   
+   return if $newgen == $currgen;
+
    g_show("Hit escape to abort...");
    my $oldsecs = time;
-   while ($currgen < $newgen) {
-      if (g_empty()) {
-         g_show("Pattern is empty.");
-         return;
-      }
-      # using g_step() as a faster method for getting to newgen
-      # is left as an exercise for the reader :)
-      g_run(1);
-      $currgen += 1;
-      g_dokey( g_getkey() );              # allow keyboard interaction
-      my $newsecs = time;
-      if ($newsecs - $oldsecs >= 1.0) {   # do an update every sec
-         $oldsecs = $newsecs;
-         g_update();
+
+   # do 1 step first, to ensure a reasonable value is saved for the step size:
+   g_run(1);
+   ++$currgen;
+
+   my $base = g_getbase();
+   my $todo = $newgen - $currgen;
+   my $exp = 0;
+   while ($todo > 0) {
+      my $num_steps = $todo % $base;
+      g_setstep($exp++);
+      $todo = ($todo - $num_steps)/$base;
+      while ($num_steps-- > 0) {
+         if (g_empty()) {
+            g_show("Pattern is empty.");
+            return;
+         }
+         g_step();
+         g_dokey( g_getkey() );              # allow keyboard interaction
+         my $newsecs = time;
+         if ($newsecs - $oldsecs >= 1.0) {   # do an update every sec
+            $oldsecs = $newsecs;
+            g_update();
+         }
       }
    }
    g_show("");
@@ -70,6 +81,18 @@ sub go_to {
 
 # --------------------------------------------------------------------
 
+sub savegen {
+   my ($filename, $gen) = @_;
+   if (not open(OUTFILE, ">".$filename)) {
+      g_warn("Can't save gen in $filename:\n$!");
+   } else {
+      print OUTFILE $gen;
+      close OUTFILE;
+   }
+}
+
+# --------------------------------------------------------------------
+
 # use same file name as in goto.py
 my $GotoINIFileName = g_getdir("data")."goto.ini";
 my $previousgen = "";
@@ -86,18 +109,14 @@ if ($gen eq "") {
    g_exit();
 } elsif ($gen eq '+' or $gen eq '-') {
    # clear the default
-   $previousgen = "";
+   savegen($GotoINIFileName, "");
 } elsif ($gen !~ /^[+-]?\d[\d,]*$/) {
    g_exit("Sorry, but \"$gen\" is not a valid integer.");
 } else {
-   $previousgen = $gen;
+   # best to save given gen now in case user aborts script
+   savegen($GotoINIFileName, $gen);
    $gen =~ s/,//g;
+   my $oldstep = g_getstep();
    go_to($gen);
-}
-
-if (not open(OUTFILE, ">".$GotoINIFileName)) {
-   g_warn("Can't save gen in $GotoINIFileName:\n$!");
-} else {
-   print OUTFILE $previousgen;
-   close OUTFILE;
+   g_setstep($oldstep);
 }
