@@ -266,6 +266,10 @@ if ($poplayer == -1) {
 }
 g_new($layername);
 
+# use same rule but without any suffix (we don't want a bounded grid)
+my ($rule, $suffix) = split(":",g_getrule(),2);
+g_setrule($rule);
+
 my ($deadr, $deadg, $deadb) = g_getcolor("deadcells");
 if (($deadr + $deadg + $deadb) / 3 > 128) {
    # use black if light background
