@@ -1,6 +1,7 @@
 # Invert all cell states in the current selection.
-# Author: Andrew Trevorrow (andrew@trevorrow.com), June 2006.
+# Author: Andrew Trevorrow (andrew@trevorrow.com), Jun 2006.
 # Updated to use exit command, Nov 2006.
+# Updated to use numstates command, Jun 2008.
 
 from glife import rect
 from time import time
@@ -10,6 +11,7 @@ r = rect( g.getselrect() )
 if r.empty: g.exit("There is no selection.")
 
 oldsecs = time()
+maxstate = g.numstates() - 1
 
 for row in xrange(r.top, r.top + r.height):
    # if large selection then give some indication of progress
@@ -22,6 +24,6 @@ for row in xrange(r.top, r.top + r.height):
    g.dokey( g.getkey() )
 
    for col in xrange(r.left, r.left + r.width):
-      g.setcell(col, row, 1 - g.getcell(col, row))
+      g.setcell(col, row, maxstate - g.getcell(col, row))
 
 if not r.visible(): g.fitsel()
