@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -37,6 +37,9 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <cstdio>
 #include <string.h>
 #include <cstdlib>
+#ifdef TIMING
+#include <sys/time.h>
+#endif
 using namespace std ;
 viewport viewport(1000, 1000) ;
 /*
@@ -48,8 +51,12 @@ public:
    virtual ~nullrender() {}
    virtual void killrect(int, int, int, int) {}
    virtual void pixblit(int, int, int, int, char*, int) {}
-   virtual void getcolors(unsigned char**, unsigned char**, unsigned char**) {}
+   virtual void getcolors(unsigned char** r, unsigned char** g, unsigned char** b) {
+      static unsigned char dummy[256];
+      *r = *g = *b = dummy;
+   }
 } ;
+
 nullrender renderer ;
 /*
  *   This is a "lifeerrors" that we can use to test some edge
@@ -69,6 +76,17 @@ public:
    virtual const char* getrulesdir() { return "Rules/" ; }
 } ;
 nullerrors nullerror ;
+#ifdef TIMING
+double start ;
+double timestamp() {
+   struct timeval tv ;
+   gettimeofday(&tv, 0) ;
+   double now = tv.tv_sec + 0.000001 * tv.tv_usec ;
+   double r = now - start ;
+   start = now ;
+   return r ;
+}
+#endif
 /*
  *   This is a "lifeerrors" that we can use to test some edge
  *   conditions.  In this case the only real thing we use
@@ -79,7 +97,11 @@ public:
    verbosestatus() {}
    virtual void fatal(const char *) {}
    virtual void warning(const char *) {}
-   virtual void status(const char *s) { cout << s << endl ; }
+   virtual void status(const char *s) {
+#ifdef TIMING
+      cout << timestamp() << " " << s << endl ;
+#endif
+   }
    virtual void beginprogress(const char *) {}
    virtual bool abortprogress(double, const char *) { return 0 ; }
    virtual void endprogress() {}
@@ -100,8 +122,9 @@ bigint maxgen = -1, inc = 0 ;
 int maxmem = 256 ;
 int hyper, render, autofit, quiet, popcount, progress ;
 int hashlife ;
-char *algoName = (char *)"QuickLife" ;
+char *algoName = 0 ;
 int verbose ;
+int timeline ;
 int stepthresh, stepfactor ;
 char *liferule = 0 ;
 char *outfilename = 0 ;
@@ -121,6 +144,7 @@ options options[] = {
   { "-o", "--output", "Output file (*.rle, *.mc, *.rle.gz, *.mc.gz)", 's',
                                                                &outfilename },
   { "-v", "--verbose", "Verbose", 'b', &verbose },
+  { "-t", "--timeline", "Use timeline", 'b', &timeline },
   { "",   "--render", "Render (benchmarking)", 'b', &render },
   { "",   "--progress", "Render during progress dialog (debugging)", 'b', &progress },
   { "",   "--popcount", "Popcount (benchmarking)", 'b', &popcount },
@@ -470,7 +494,7 @@ void runtestscript(const char *testscript) {
 }
 int main(int argc, char *argv[]) {
    cout << 
-    "This is bgolly " STRINGIFY(VERSION) " Copyright 2009 The Golly Gang."
+    "This is bgolly " STRINGIFY(VERSION) " Copyright 2010 The Golly Gang."
                                                             << endl << flush ;
    qlifealgo::doInitializeAlgoInfo(staticAlgoInfo::tick()) ;
    hlifealgo::doInitializeAlgoInfo(staticAlgoInfo::tick()) ;
@@ -542,6 +566,8 @@ case 's':
       if (strlen(outfilename) > 200)
          lifefatal("Output filename too long") ;
    }
+   if (timeline && hyper)
+      lifefatal("Cannot use both timeline and hyperthreading") ;
    imp = createUniverse() ;
    if (progress)
      lifeerrors::seterrorhandler(&nullerror) ;
@@ -550,6 +576,9 @@ case 's':
      hlifealgo::setVerbose(1) ;
    }
    imp->setMaxMemory(maxmem) ;
+#ifdef TIMING
+   timestamp() ;
+#endif
    if (testscript) {
      if (argc > 1) {
        filename = argv[1] ;
@@ -567,6 +596,15 @@ case 's':
       imp->setrule(liferule) ;
    if (inc != 0)
       imp->setIncrement(inc) ;
+   if (timeline) {
+      int lowbit = inc.lowbitset() ;
+      bigint t = 1 ;
+      for (int i=0; i<lowbit; i++)
+         t.mul_smallint(2) ;
+      if (t != inc)
+         lifefatal("Bad increment for timeline") ;
+      imp->startrecording(2, lowbit) ;
+   }
    int fc = 0 ;
    for (;;) {
       if (quiet < 2) {
@@ -595,6 +633,8 @@ case 's':
       imp->step() ;
       if (maxgen < 0 && outfilename != 0)
          writepat(fc++) ;
+      if (timeline && imp->getframecount() + 2 > MAX_FRAME_COUNT)
+         imp->pruneframes() ;
       if (hyper)
          imp->setIncrement(imp->getGeneration()) ;
    }
