@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -372,6 +372,10 @@ void bigint::ensurework(int sz) const {
 }
 void bigint::mul_smallint(int a) {
    int c ;
+   if (a == 0) {
+      *this = 0 ;
+      return ;
+   }
    if (v.i & 1) {
       c = (v.i >> 1) * a ;
       if (c / a == (v.i >> 1)) {
