@@ -1,7 +1,7 @@
                        /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -449,25 +449,6 @@ ghashbase::ghashbase() {
    cacheinvalid = 0 ;
    gccount = 0 ;
    gcstep = 0 ;
-   /* AKT: can't do this here (it would clobber user's preferred colors
-      if/when we let people override the default colors)
-   for (int i=0; i<256; i++)
-     cellred[i] = cellgreen[i] = cellblue[i] = 0 ;
-   cellred[1] |= 0x92 ;
-   cellblue[2] |= 0x92 ;
-   cellgreen[4] |= 0x92 ;
-   cellred[8] |= 0x49 ;
-   cellblue[16] |= 0x49 ;
-   cellgreen[32] |= 0x49 ;
-   cellred[64] |= 0x24 ;
-   cellgreen[128] |= 0x24 ;
-   for (int i=1; i<256; i++) {
-     cellred[i] = cellred[i&-i] | cellred[i&(i-1)] ;
-     cellgreen[i] = cellgreen[i&-i] | cellgreen[i&(i-1)] ;
-     cellblue[i] = cellblue[i&-i] | cellblue[i&(i-1)] ;
-   }
-   cellred[0] = cellgreen[0] = cellblue[0] = 255 ;
-   */
 }
 /**
  *   Destructor frees memory.
@@ -537,6 +518,14 @@ void ghashbase::step() {
       root = newroot ;
    }
    depth = ghnode_depth(root) ;
+   extendtimeline() ;
+}
+void ghashbase::setcurrentstate(void *n) {
+   if (root != (ghnode *)n) {
+      root = (ghnode *)n ;
+      depth = ghnode_depth(root) ;
+      popValid = 0 ;
+   }
 }
 /*
  *   Set the max memory
@@ -670,7 +659,14 @@ ghnode *ghashbase::setbit(ghnode *n, int x, int y, int newstate, int depth) {
           l->ne = (state)newstate ;
       return (ghnode *)l ;
    } else {
-      int w = 1 << depth ;
+      unsigned int w = 0, wh = 0 ;
+      if (depth >= 31) {
+         if (depth == this->depth)
+            wh = 0x80000000 ;
+      } else {
+         w = 1 << depth ;
+         wh = 1 << (depth - 1) ;
+      }
       if (depth > 31)
          w = 0 ;
       depth-- ;
@@ -692,8 +688,8 @@ ghnode *ghashbase::setbit(ghnode *n, int x, int y, int newstate, int depth) {
          else
             *nptr = newclearedghnode() ;
       }
-      ghnode *s = setbit(*nptr, (x & (w - 1)) - (w >> 1),
-                         (y & (w - 1)) - (w >> 1), newstate, depth) ;
+      ghnode *s = setbit(*nptr, (x & (w - 1)) - wh,
+                         (y & (w - 1)) - wh, newstate, depth) ;
       if (hashed) {
          ghnode *nw = n->nw ;
          ghnode *sw = n->sw ;
@@ -736,9 +732,14 @@ int ghashbase::getbit(ghnode *n, int x, int y, int depth) {
          else
            return l->ne ;
    } else {
-      int w = 1 << depth ;
-      if (depth > 31)
-         w = 0 ;
+      unsigned int w = 0, wh = 0 ;
+      if (depth >= 31) {
+         if (depth == this->depth)
+            wh = 0x80000000 ;
+      } else {
+         w = 1 << depth ;
+         wh = 1 << (depth - 1) ;
+      }
       ghnode *nptr ;
       depth-- ;
       if (x < 0) {
@@ -754,7 +755,7 @@ int ghashbase::getbit(ghnode *n, int x, int y, int depth) {
       }
       if (nptr == 0 || nptr == zeroghnode(depth))
          return 0 ;
-      return getbit(nptr, (x & (w - 1)) - (w >> 1), (y & (w - 1)) - (w >> 1),
+      return getbit(nptr, (x & (w - 1)) - wh, (y & (w - 1)) - wh,
                     depth) ;
    }
 }
@@ -789,8 +790,8 @@ int ghashbase::nextbit(ghnode *n, int x, int y, int depth, int &v) {
       }
       return -1 ; // none found
    } else {
-      int w = 1 << depth ;
-      int wh = 1 << (depth - 1) ;
+      unsigned int w = 1 << depth ;
+      unsigned int wh = w >> 1 ;
       ghnode *lft, *rght ;
       depth-- ;
       if (y < 0) {
@@ -1146,8 +1147,10 @@ void ghashbase::do_gc(int invalidate) {
       gc_mark(zeroghnodea[i], 0) ; // never invalidate zeroghnode
    for (i=0; i<gsp; i++) {
       poller->poll() ;
-      gc_mark(stack[i], invalidate) ;
+      gc_mark((ghnode *)stack[i], invalidate) ;
    }
+   for (int i=0; i<timeline.framecount; i++)
+      gc_mark((ghnode *)timeline.frames[i], invalidate) ;
    hashpop = 0 ;
    memset(hashtab, 0, sizeof(ghnode *) * hashprime) ;
    freeghnodes = 0 ;
@@ -1369,6 +1372,75 @@ const char *ghashbase::readmacrocell(char *line) {
             *pp = 0 ;
             generation = bigint(p) ;
             break ;
+	    // either:
+	    //   #FRAMES count base inc
+	    // or
+	    //   #FRAME index node
+	 case 'F':
+	    if (strncmp(line, "#FRAMES ", 8) == 0) {
+	       p = line + 8 ;
+	       while (*p && *p <= ' ')
+		 p++ ;
+	       int cnt = atol(p) ;
+	       if (cnt < 0 || cnt > MAX_FRAME_COUNT)
+		  return "Bad FRAMES line" ;
+	       destroytimeline() ;
+	       while ('0' <= *p && *p <= '9')
+		 p++ ;
+	       while (*p && *p <= ' ')
+		 p++ ;
+	       pp = p ;
+	       while (*pp >= '0' && *pp <= '9') pp++ ;
+	       if (*pp == 0)
+		  return "Bad FRAMES line" ;
+	       *pp = 0 ;
+	       timeline.start = bigint(p) ;
+	       timeline.end = timeline.start ;
+	       timeline.next = timeline.end ;
+	       p = pp + 1 ;
+	       while (*p && *p <= ' ')
+		 p++ ;
+	       pp = p ;
+	       while (*pp > ' ')
+                  pp++ ;
+	       *pp = 0 ;
+               if (strchr(p, '^')) {
+                  int tbase=0, texpo=0 ;
+                  if (sscanf(p, "%d^%d", &tbase, &texpo) != 2 ||
+                      tbase < 2 || texpo < 0)
+                     return "Bad FRAMES line" ;
+                  timeline.base = tbase ;
+                  timeline.expo = texpo ;
+                  timeline.inc = 1 ;
+                  while (texpo--)
+                     timeline.inc.mul_smallint(tbase) ;
+               } else {
+                  timeline.inc = bigint(p) ;
+                  // if it's a power of two, we're good
+                  int texpo = timeline.inc.lowbitset() ;
+                  int tbase = 2 ;
+                  bigint test = 1 ;
+                  for (int i=0; i<texpo; i++)
+                     test += test ;
+                  if (test != timeline.inc)
+                     return "Bad increment (missing ^) in FRAMES" ;
+                  timeline.base = tbase ;
+                  timeline.expo = texpo ;
+               }
+	    } else if (strncmp(line, "#FRAME ", 7) == 0) {
+	       int frameind = 0 ;
+	       g_uintptr_t nodeind = 0 ;
+	       n = sscanf(line+7, "%d %" PRIuPTR, &frameind, &nodeind) ;
+	       if (n != 2 || frameind > MAX_FRAME_COUNT || frameind < 0 ||
+		   nodeind < 0 || nodeind > i ||
+		   timeline.framecount != frameind)
+		  return "Bad FRAME line" ;
+	       timeline.frames.push_back(ind[nodeind]) ;
+	       timeline.framecount++ ;
+	       timeline.end = timeline.next ;
+	       timeline.next += timeline.inc ;
+	    }
+	    break ;
          }
       } else {
          n = sscanf(line, "%d %" PRIuPTR " %" PRIuPTR " %" PRIuPTR " %" PRIuPTR " %d", &d, &nw, &ne, &sw, &se, &r) ;
@@ -1513,7 +1585,7 @@ g_uintptr_t ghashbase::writecell_2p2(FILE *f, ghnode *root, int depth) {
       thiscell = ++cellcounter ;
       if ((cellcounter & 4095) == 0) {
          unsigned long siz = ftell(f) ;
-         sprintf(progressmsg, "File size: %.2g MB", double((siz >> 20))) ;
+         sprintf(progressmsg, "File size: %.2f MB", double(siz) / 1048576.0) ;
          lifeabortprogress(thiscell/(double)writecells, progressmsg) ;
       }
       ghleaf *n = (ghleaf *)root ;
@@ -1534,7 +1606,7 @@ g_uintptr_t ghashbase::writecell_2p2(FILE *f, ghnode *root, int depth) {
       thiscell = ++cellcounter ;
       if ((cellcounter & 4095) == 0) {
          unsigned long siz = ftell(f) ;
-         sprintf(progressmsg, "File size: %.2g MB", double((siz >> 20))) ;
+         sprintf(progressmsg, "File size: %.2f MB", double(siz) / 1048576.0) ;
          lifeabortprogress(thiscell/(double)writecells, progressmsg) ;
       }
       root->next = (ghnode *)thiscell ;
@@ -1549,8 +1621,7 @@ const char *ghashbase::writeNativeFormat(FILE *f, char *comments) {
    fputs("[M2] (golly " STRINGIFY(VERSION) ")", f) ;
    fputs("\n", f) ;
    
-   // AKT: always write out explicit rule, and avoid global_liferules
-   // because not all algos use it
+   // AKT: always write out explicit rule
    fprintf(f, "#R %s\n", getrule()) ;
    
    if (generation > bigint::zero) {
@@ -1568,11 +1639,41 @@ const char *ghashbase::writeNativeFormat(FILE *f, char *comments) {
    */
    /* this is the new two-pass way */
    cellcounter = 0 ;
+   vector<int> depths(timeline.framecount) ;
+   int framestosave = timeline.framecount ;
+   if (timeline.savetimeline == 0)
+     framestosave = 0 ;
+   if (framestosave) {
+     for (int i=0; i<timeline.framecount; i++) {
+       ghnode *frame = (ghnode*)timeline.frames[i] ;
+       depths[i] = ghnode_depth(frame) ;
+     }
+     for (int i=0; i<timeline.framecount; i++) {
+       ghnode *frame = (ghnode*)timeline.frames[i] ;
+       writecell_2p1(frame, depths[i]) ;
+     }
+   }
    writecell_2p1(root, depth) ;
    writecells = cellcounter ;
    cellcounter = 0 ;
+   if (framestosave) {
+     fprintf(f, "#FRAMES %d ", timeline.framecount) ;
+     fprintf(f, "%s ", timeline.start.tostring()) ;
+     fprintf(f, "%d^%d\n", timeline.base, timeline.expo) ;
+     for (int i=0; i<timeline.framecount; i++) {
+       ghnode *frame = (ghnode*)timeline.frames[i] ;
+       writecell_2p2(f, frame, depths[i]) ;
+       fprintf(f, "#FRAME %d %" PRIuPTR "\n", i, (g_uintptr_t)(frame->next)) ;
+     }
+   }
    writecell_2p2(f, root, depth) ;
    /* end new two-pass way */
+   if (framestosave) {
+     for (int i=0; i<timeline.framecount; i++) {
+       ghnode *frame = (ghnode*)timeline.frames[i] ;
+       aftercalcpop2(frame, depths[i], 0) ;
+     }
+   }
    aftercalcpop2(root, depth, 0) ;
    inGC = 0 ;
    return 0 ;
