@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -555,6 +555,7 @@ hlifealgo::hlifealgo() {
    cacheinvalid = 0 ;
    gccount = 0 ;
    gcstep = 0 ;
+   serial = 0 ;
 }
 /**
  *   Destructor frees memory.
@@ -624,6 +625,14 @@ void hlifealgo::step() {
       root = newroot ;
    }
    depth = node_depth(root) ;
+   extendtimeline() ;
+}
+void hlifealgo::setcurrentstate(void *n) {
+   if (root != (node *)n) {
+      root = (node *)n ;
+      depth = node_depth(root) ;
+      popValid = 0 ;
+   }
 }
 /*
  *   Set the max memory
@@ -783,9 +792,14 @@ node *hlifealgo::setbit(node *n, int x, int y, int newstate, int depth) {
       }
       return (node *)l ;
    } else {
-      int w = 1 << depth ;
-      if (depth > 31)
-         w = 0 ;
+      unsigned int w = 0, wh = 0 ;
+      if (depth >= 31) {
+         if (depth == this->depth)
+            wh = 0x80000000 ;
+      } else {
+         w = 1 << depth ;
+         wh = 1 << (depth - 1) ;
+      }
       depth-- ;
       node **nptr ;
       if (x < 0) {
@@ -805,8 +819,8 @@ node *hlifealgo::setbit(node *n, int x, int y, int newstate, int depth) {
          else
             *nptr = newclearednode() ;
       }
-      node *s = setbit(*nptr, (x & (w - 1)) - (w >> 1),
-                              (y & (w - 1)) - (w >> 1), newstate, depth) ;
+      node *s = setbit(*nptr, (x & (w - 1)) - wh,
+                              (y & (w - 1)) - wh, newstate, depth) ;
       if (hashed) {
          node *nw = n->nw ;
          node *sw = n->sw ;
@@ -853,11 +867,16 @@ int hlifealgo::getbit(node *n, int x, int y, int depth) {
          return 1 ;
       return 0 ;
    } else {
-      int w = 1 << depth ;
-      if (depth > 31)
-         w = 0 ;
-      node *nptr ;
+      unsigned int w = 0, wh = 0 ;
+      if (depth >= 31) {
+         if (depth == this->depth)
+            wh = 0x80000000 ;
+      } else {
+         w = 1 << depth ;
+         wh = 1 << (depth - 1) ;
+      }
       depth-- ;
+      node *nptr ;
       if (x < 0) {
          if (y < 0)
             nptr = n->sw ;
@@ -871,8 +890,7 @@ int hlifealgo::getbit(node *n, int x, int y, int depth) {
       }
       if (nptr == 0 || nptr == zeronode(depth))
          return 0 ;
-      return getbit(nptr, (x & (w - 1)) - (w >> 1), (y & (w - 1)) - (w >> 1),
-                    depth) ;
+      return getbit(nptr, (x & (w - 1)) - wh, (y & (w - 1)) - wh, depth) ;
    }
 }
 /*
@@ -904,8 +922,9 @@ int hlifealgo::nextbit(node *n, int x, int y, int depth) {
       }
       return -1 ; // none found
    } else {
-      int w = 1 << depth ;
-      int wh = 1 << (depth - 1) ;
+      unsigned int w = 0, wh = 0 ;
+      w = 1 << depth ;
+      wh = 1 << (depth - 1) ;
       node *lft, *rght ;
       depth-- ;
       if (y < 0) {
@@ -1264,6 +1283,8 @@ void hlifealgo::do_gc(int invalidate) {
       poller->poll() ;
       gc_mark(stack[i], invalidate) ;
    }
+   for (int i=0; i<timeline.framecount; i++)
+      gc_mark((node *)timeline.frames[i], invalidate) ;
    hashpop = 0 ;
    memset(hashtab, 0, sizeof(node *) * hashprime) ;
    freenodes = 0 ;
@@ -1293,7 +1314,8 @@ void hlifealgo::do_gc(int invalidate) {
    inGC = 0 ;
    if (verbose) {
      int perc = freed_nodes / (totalthings / 100) ;
-     sprintf(statusline+strlen(statusline), " freed %d percent.", perc) ;
+     sprintf(statusline+strlen(statusline), " freed %d percent (%d).",
+                                                   perc, (int)freed_nodes) ;
      lifestatus(statusline) ;
    }
    if (needPop) {
@@ -1504,7 +1526,7 @@ default:       return "Illegal character in readmacrocell." ;
             *pp = 0 ;
             
             // AKT: need to check for B0-not-S8 rule
-            err = global_liferules.setrule(p);
+            err = global_liferules.setrule(p, this);
             if (err)
                return err;
             if (global_liferules.hasB0notS8)
@@ -1519,6 +1541,75 @@ default:       return "Illegal character in readmacrocell." ;
             *pp = 0 ;
             generation = bigint(p) ;
             break ;
+	    // either:
+	    //   #FRAMES count base inc
+	    // or
+	    //   #FRAME index node
+	 case 'F':
+	    if (strncmp(line, "#FRAMES ", 8) == 0) {
+	       p = line + 8 ;
+	       while (*p && *p <= ' ')
+		 p++ ;
+	       int cnt = atol(p) ;
+	       if (cnt < 0 || cnt > MAX_FRAME_COUNT)
+		  return "Bad FRAMES line" ;
+	       destroytimeline() ;
+	       while ('0' <= *p && *p <= '9')
+		 p++ ;
+	       while (*p && *p <= ' ')
+		 p++ ;
+	       pp = p ;
+	       while (*pp >= '0' && *pp <= '9') pp++ ;
+	       if (*pp == 0)
+		  return "Bad FRAMES line" ;
+	       *pp = 0 ;
+	       timeline.start = bigint(p) ;
+	       timeline.end = timeline.start ;
+               timeline.next = timeline.start ;
+	       p = pp + 1 ;
+	       while (*p && *p <= ' ')
+		 p++ ;
+	       pp = p ;
+	       while (*pp > ' ')
+                  pp++ ;
+	       *pp = 0 ;
+               if (strchr(p, '^')) {
+                  int tbase=0, texpo=0 ;
+                  if (sscanf(p, "%d^%d", &tbase, &texpo) != 2 ||
+                      tbase < 2 || texpo < 0)
+                     return "Bad FRAMES line" ;
+                  timeline.base = tbase ;
+                  timeline.expo = texpo ;
+                  timeline.inc = 1 ;
+                  while (texpo--)
+                     timeline.inc.mul_smallint(tbase) ;
+               } else {
+	          timeline.inc = bigint(p) ;
+                  // if it's a power of two, we're good
+                  int texpo = timeline.inc.lowbitset() ;
+                  int tbase = 2 ;
+                  bigint test = 1 ;
+                  for (int i=0; i<texpo; i++)
+                     test += test ;
+                  if (test != timeline.inc)
+                     return "Bad increment (missing ^) in FRAMES" ;
+                  timeline.base = tbase ;
+                  timeline.expo = texpo ;
+               }
+	    } else if (strncmp(line, "#FRAME ", 7) == 0) {
+	       int frameind = 0 ;
+	       g_uintptr_t nodeind = 0 ;
+	       n = sscanf(line+7, "%d %" PRIuPTR, &frameind, &nodeind) ;
+	       if (n != 2 || frameind > MAX_FRAME_COUNT || frameind < 0 ||
+		   nodeind < 0 || nodeind > i ||
+		   timeline.framecount != frameind)
+		  return "Bad FRAME line" ;
+	       timeline.frames.push_back(ind[nodeind]) ;
+	       timeline.framecount++ ;
+	       timeline.end = timeline.next ;
+	       timeline.next += timeline.inc ;
+	    }
+	    break ;
          }
       } else {
          n = sscanf(line, "%d %" PRIuPTR " %" PRIuPTR " %" PRIuPTR " %" PRIuPTR " %d", &d, &nw, &ne, &sw, &se, &r) ;
@@ -1565,15 +1656,23 @@ default:       return "Illegal character in readmacrocell." ;
 }
 const char *hlifealgo::setrule(const char *s) {
    poller->bailIfCalculating() ;
+   const char* err = global_liferules.setrule(s, this);
+   if (err) return err;
+   if (serial == global_liferules.getSerial())
+      return 0 ;
+
+   serial = global_liferules.getSerial() ;
    clearcache() ;
    
-   // AKT: need to check for B0-not-S8 rule
-   const char* err = global_liferules.setrule(s);
-   if (err)
-      return err;
+   
    if (global_liferules.hasB0notS8)
       return "B0-not-S8 rules are not allowed in HashLife.";
-   
+      
+   if (global_liferules.isHexagonal())
+      grid_type = HEX_GRID;
+   else
+      grid_type = SQUARE_GRID;
+      
    return 0 ;
 }
 void hlifealgo::unpack8x8(unsigned short nw, unsigned short ne,
@@ -1685,7 +1784,7 @@ g_uintptr_t hlifealgo::writecell_2p2(FILE *f, node *root, int depth) {
       thiscell = ++cellcounter ;
       if ((cellcounter & 4095) == 0) {
          unsigned long siz = ftell(f) ;
-         sprintf(progressmsg, "File size: %.2g MB", double((siz >> 20))) ;
+         sprintf(progressmsg, "File size: %.2f MB", double(siz) / 1048576.0) ;
          lifeabortprogress(thiscell/(double)writecells, progressmsg) ;
       }
       int i, j ;
@@ -1720,7 +1819,7 @@ g_uintptr_t hlifealgo::writecell_2p2(FILE *f, node *root, int depth) {
       thiscell = ++cellcounter ;
       if ((cellcounter & 4095) == 0) {
          unsigned long siz = ftell(f) ;
-         sprintf(progressmsg, "File size: %.2g MB", double((siz >> 20))) ;
+         sprintf(progressmsg, "File size: %.2f MB", double(siz) / 1048576.0) ;
          lifeabortprogress(thiscell/(double)writecells, progressmsg) ;
       }
       root->next = (node *)thiscell ;
@@ -1734,10 +1833,10 @@ const char *hlifealgo::writeNativeFormat(FILE *f, char *comments) {
    int depth = node_depth(root) ;
    fputs("[M2] (golly " STRINGIFY(VERSION) ")", f) ;
    fputs("\n", f) ;
-   if (!global_liferules.isRegularLife()) {
-      // write non-Life rule
-      fprintf(f, "#R %s\n", global_liferules.getrule()) ;
-   }
+
+   // AKT: always write out explicit rule
+   fprintf(f, "#R %s\n", global_liferules.getrule()) ;
+   
    if (generation > bigint::zero) {
       // write non-zero gen count
       fprintf(f, "#G %s\n", generation.tostring('\0')) ;
@@ -1753,11 +1852,41 @@ const char *hlifealgo::writeNativeFormat(FILE *f, char *comments) {
    */
    /* this is the new two-pass way */
    cellcounter = 0 ;
+   vector<int> depths(timeline.framecount) ;
+   int framestosave = timeline.framecount ;
+   if (timeline.savetimeline == 0)
+     framestosave = 0 ;
+   if (framestosave) {
+     for (int i=0; i<timeline.framecount; i++) {
+       node *frame = (node*)timeline.frames[i] ;
+       depths[i] = node_depth(frame) ;
+     }
+     for (int i=0; i<timeline.framecount; i++) {
+       node *frame = (node*)timeline.frames[i] ;
+       writecell_2p1(frame, depths[i]) ;
+     }
+   }
    writecell_2p1(root, depth) ;
    writecells = cellcounter ;
    cellcounter = 0 ;
+   if (framestosave) {
+     fprintf(f, "#FRAMES %d ", timeline.framecount) ;
+     fprintf(f, "%s ", timeline.start.tostring()) ;
+     fprintf(f, "%d^%d\n", timeline.base, timeline.expo) ;
+     for (int i=0; i<timeline.framecount; i++) {
+       node *frame = (node*)timeline.frames[i] ;
+       writecell_2p2(f, frame, depths[i]) ;
+       fprintf(f, "#FRAME %d %" PRIuPTR "\n", i, (g_uintptr_t)(frame->next)) ;
+     }
+   }
    writecell_2p2(f, root, depth) ;
    /* end new two-pass way */
+   if (framestosave) {
+     for (int i=0; i<timeline.framecount; i++) {
+       node *frame = (node*)timeline.frames[i] ;
+       aftercalcpop2(frame, depths[i], 0) ;
+     }
+   }
    aftercalcpop2(root, depth, 0) ;
    inGC = 0 ;
    return 0 ;
