@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -169,9 +169,7 @@ public:
    virtual void setIncrement(bigint inc) ;
    virtual void setIncrement(int inc) { setIncrement(bigint(inc)) ; }
    virtual void setGeneration(bigint gen) { generation = gen ; }
-   virtual const bigint &getIncrement() { return increment ; }
    virtual const bigint &getPopulation() ;
-   virtual const bigint &getGeneration() { return generation ; }
    virtual int isEmpty() ;
    virtual int hyperCapable() { return 1 ; }
    virtual void setMaxMemory(int m) ;
@@ -179,6 +177,8 @@ public:
    virtual const char *setrule(const char *s) ;
    virtual const char *getrule() { return global_liferules.getrule() ; }
    virtual void step() ;
+   virtual void* getcurrentstate() { return root ; }
+   virtual void setcurrentstate(void *n) ;
    /*
     *   The contract of draw() is that it render every pixel in the
     *   viewport precisely once.  This allows us to eliminate all
@@ -225,9 +225,7 @@ private:
    g_uintptr_t totalthings ;
    node *nodeblocks ;
    char *ruletable ;
-   bigint generation ;
    bigint population ;
-   bigint increment ;
    bigint setincrement ;
    bigint pow2step ; // greatest power of two in increment
    int nonpow2 ; // increment / pow2step
@@ -249,6 +247,7 @@ private:
    int gccount ; // how many gcs total this pattern
    int gcstep ; // how many gcs this step
    static char statusline[] ;
+   int serial ; // serialization from global liferules
 //
    void leafres(leaf *n) ;
    void resize() ;
