@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -23,11 +23,12 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
                         / ***/
 #include "jvnalgo.h"
 
-// AKT: for case-insensitive string comparison
+// for case-insensitive string comparison
 #include <string.h>
 #ifndef WIN32
    #define stricmp strcasecmp
 #endif
+#include <string>
 
 using namespace std ;
 
@@ -39,23 +40,55 @@ int jvnalgo::NumCellStates() {
    return N_STATES[current_rule];
 }
 
-const char* jvnalgo::setrule(const char *s) 
+const char* jvnalgo::setrule(const char *s)
 {
+   const char* colonptr = strchr(s,':');
+   string rule_name(s);
+   if(colonptr)
+      rule_name.assign(s,colonptr);
+   
    // check the requested string against the named rules, and deprecated versions
-   if (stricmp(s, RULE_STRINGS[JvN29]) == 0 || stricmp(s, "JvN-29") == 0)
+   if (stricmp(rule_name.c_str(), RULE_STRINGS[JvN29]) == 0 || 
+        stricmp(rule_name.c_str(), "JvN-29") == 0)
       current_rule = JvN29;
-   else if (stricmp(s, RULE_STRINGS[Nobili32]) == 0 || stricmp(s, "JvN-32") == 0)
+   else if (stricmp(rule_name.c_str(), RULE_STRINGS[Nobili32]) == 0 || 
+        stricmp(rule_name.c_str(), "JvN-32") == 0)
       current_rule = Nobili32;
-   else if (stricmp(s, RULE_STRINGS[Hutton32]) == 0 || stricmp(s, "modJvN-32") == 0)
+   else if (stricmp(rule_name.c_str(), RULE_STRINGS[Hutton32]) == 0 || 
+        stricmp(rule_name.c_str(), "modJvN-32") == 0)
       current_rule = Hutton32;
-   else return "This algorithm only supports these rules: JvN29, Nobili32, Hutton32.";
+   else {
+      return "This algorithm only supports these rules: JvN29, Nobili32, Hutton32.";
+   }
+   
+   // check for rule suffix like ":T200,100" to specify a bounded universe
+   if (colonptr) {
+      const char* err = setgridsize(colonptr);
+      if (err) return err;
+   } else {
+      // universe is unbounded
+      gridwd = 0;
+      gridht = 0;
+   }
+
    maxCellStates = N_STATES[current_rule];
-   ghashbase::setrule(RULE_STRINGS[current_rule]) ;
+   ghashbase::setrule(RULE_STRINGS[current_rule]);
    return NULL;
 }
 
 const char* jvnalgo::getrule() {
-   return RULE_STRINGS[current_rule];
+   // return canonical rule string
+   static char canonrule[MAXRULESIZE];
+   sprintf(canonrule, "%s", RULE_STRINGS[current_rule]);
+   if (gridwd > 0 || gridht > 0) {
+      // setgridsize() was successfully called above, so append suffix
+      int len = strlen(canonrule);
+      const char* bounds = canonicalsuffix();
+      int i = 0;
+      while (bounds[i]) canonrule[len++] = bounds[i++];
+      canonrule[len] = 0;
+   }
+   return canonrule;
 }
 
 const char* jvnalgo::DefaultRule() {
