@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -35,10 +35,30 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include "lifepoll.h"
 #include "readpattern.h"
 #include "platform.h"
+// moving the include vector *before* platform.h breaks compilation
+#include <vector>
+// this must not be increased beyond 32767, because we use a bigint
+// multiply that only supports multiplicands up to that size.
+const int MAX_FRAME_COUNT = 32000 ;
+
+/**
+ *   Timeline support is pretty generic.
+ */
+class timeline_t {
+public:
+   timeline_t() : recording(0), framecount(0), savetimeline(1),
+                  start(0), inc(0), next(0), end(0), frames() {}
+   int recording, framecount, base, expo, savetimeline ;
+   bigint start, inc, next, end ;
+   vector<void *> frames ;
+} ;
 
 class lifealgo {
 public:
-   lifealgo() { poller = &default_poller ; }
+   lifealgo() : generation(0), increment(0), timeline(), grid_type(SQUARE_GRID)
+      {  poller = &default_poller ;
+         gridwd = gridht = 0 ;         // default is an unbounded universe
+      }
    virtual void clearall() = 0 ;
    // returns <0 if error
    virtual int setcell(int x, int y, int newstate) = 0 ;
@@ -49,9 +69,9 @@ public:
    virtual void setIncrement(bigint inc) = 0 ;
    virtual void setIncrement(int inc) = 0 ;
    virtual void setGeneration(bigint gen) = 0 ;
-   virtual const bigint &getIncrement() = 0 ;
+   const bigint &getIncrement() { return increment ; }
+   const bigint &getGeneration() { return generation ; }
    virtual const bigint &getPopulation() = 0 ;
-   virtual const bigint &getGeneration() = 0 ;
    virtual int isEmpty() = 0 ;
    // can we do the gen count doubling? only hashlife
    virtual int hyperCapable() = 0 ;
@@ -73,13 +93,57 @@ public:
    // into some global shared thing or something rather than use static.
    static void setVerbose(int v) { verbose = v ; }
    static int getVerbose() { return verbose ; }
+
    virtual const char* DefaultRule() { return "B3/S23"; }
    // return number of cell states in this universe (2..256)
    virtual int NumCellStates() { return 2; }
+
+   // timeline support
+   virtual void* getcurrentstate() = 0 ;
+   virtual void setcurrentstate(void *) = 0 ;
+   int startrecording(int base, int expo) ;
+   pair<int, int> stoprecording() ;
+   pair<int, int> getbaseexpo()
+                       { return make_pair(timeline.base, timeline.expo) ; }
+   void extendtimeline() ;
+   void pruneframes() ;
+   const bigint &gettimelinestart() { return timeline.start ; }
+   const bigint &gettimelineend() { return timeline.end ; }
+   const bigint &gettimelineinc() { return timeline.inc ; }
+   const int getframecount() { return timeline.framecount ; }
+   const int isrecording() { return timeline.recording ; }
+   int gotoframe(int i) ;
+   void destroytimeline() ;
+   void savetimelinewithframe(int yesno) { timeline.savetimeline = yesno ; }
+
+   // support for a bounded universe with various topologies:
+   // plane, cylinder, torus, Klein bottle, cross-surface, sphere
+   unsigned int gridwd, gridht ;    // bounded universe if either is > 0
+   bigint gridleft, gridright ;     // undefined if gridwd is 0
+   bigint gridtop, gridbottom ;     // undefined if gridht is 0
+   bool boundedplane ;              // topology is a bounded plane?
+   bool sphere ;                    // topology is a sphere?
+   bool htwist, vtwist ;            // Klein bottle if either is true,
+                                    // or cross-surface if both are true
+   int hshift, vshift ;             // torus with horizontal or vertical shift
+   const char* setgridsize(const char* suffix) ;
+   // use in setrule() to parse a suffix like ":T100,200" and set
+   // the above parameters
+   const char* canonicalsuffix() ;
+   // use in setrule() to return the canonical version of suffix;
+   // eg. ":t0020" would be converted to ":T20,0"
+   
+   enum TGridType { SQUARE_GRID, TRI_GRID, HEX_GRID } ;
+   TGridType getgridtype() const { return grid_type ; }
+
 protected:
    lifepoll *poller ;
    static int verbose ;
    int maxCellStates ; // keep up to date; setcell depends on it
+   bigint generation ;
+   bigint increment ;
+   timeline_t timeline ;
+   TGridType grid_type ;
 } ;
 
 /**
