@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -188,6 +188,8 @@ void qlifealgo::uproot() {
    bmax <<= 3 ;
    bmax -= 121 ;
    minlow32 = 8 * minlow32 - 4 ;
+   if (rootlev >= 38)
+     lifefatal("internal:  push too deep for qlifealgo") ;
    for (int i=0; i<2; i++) {
      supertile *oroot = root ;
      rootlev++ ;
@@ -229,8 +231,9 @@ void qlifealgo::clearall() {
       free(memused) ;
       memused = nu ;
    }
+   serial = 0 ;
    generation = 0 ;
-   stepsize = 1 ;
+   increment = 1 ;
    tilelist = 0 ;
    supertilelist = 0 ;
    bricklist = 0 ;
@@ -651,22 +654,38 @@ int qlifealgo::p10(tile *plu, tile *pu, tile *pl, tile *p) {
       return i ? ((i & 0x100) >> 7) | 1 : 0 ;
 }
 /**
- *   Mark a node and its subnodes as needing recomputation.
+ *   Mark a node and its subnodes as changed.  We really
+ *   only mark those nodes that have any cells set at all.
  */
-void qlifealgo::markglobalchange(supertile *p, int lev) {
+int qlifealgo::markglobalchange(supertile *p, int lev) {
    int i ;
    if (lev == 0) {
       tile *pp = (tile *)p ;
       if (pp != emptytile) {
-         pp->c[0] = pp->c[5] = 0x1ff ;
-         pp->c[1] = pp->c[2] = pp->c[3] = pp->c[4] = 0x3ff ;
+        int s = 0 ;
+        for (int i=0; i<4; i++)
+          for (int j=0; j<16; j++)
+            s |= pp->b[i]->d[j] ;
+        if (s) {
+          pp->c[0] = pp->c[5] = 0x1ff ;
+          pp->c[1] = pp->c[2] = pp->c[3] = pp->c[4] = 0x3ff ;
+          return 0x603 ;
+        }
       }
+      return 0 ;
    } else {
       if (p != nullroots[lev]) {
-         p->flags |= 0xfffffff ;
-         for (i=0; i<8; i++)
-            markglobalchange(p->d[i], lev-1) ;
+         int nchanging = 0 ;
+         if (generation.odd())
+           for (i=0; i<8; i++)
+              nchanging |= (markglobalchange(p->d[i], lev-1)) << i ;
+         else
+           for (i=0; i<8; i++)
+             nchanging |= (markglobalchange(p->d[i], lev-1)) << (7 - i) ;
+         p->flags |= nchanging | 0xf0000000 ;
+         return upchanging(nchanging) ;
       }
+      return 0 ;
    }
 }
 void qlifealgo::markglobalchange() {
@@ -1083,8 +1102,12 @@ void qlifealgo::dogen() {
 #ifdef STATS
    ds = 0 ; dq = 0 ; rcc = 0 ;
 #endif
-   while (uproot_needed())
-      uproot() ;
+   // AKT: if grid is bounded then we should never need to call uproot() here
+   // because setrule() has already expanded the universe to enclose the grid
+   if (gridwd == 0 || gridht == 0) {
+      while (uproot_needed())
+         uproot() ;
+   }
    if (generation.odd())
       doquad10(root, nullroot, nullroot, nullroot, rootlev) ;
    else
@@ -1105,7 +1128,7 @@ void qlifealgo::dogen() {
  */
 void qlifealgo::step() {
    poller->bailIfCalculating() ;
-   bigint t = stepsize ;
+   bigint t = increment ;
    while (t != 0) {
       // AKT: emulate B0-not-S8 rule by changing rule table depending on gen parity
       if (global_liferules.hasB0notS8) {
@@ -1120,17 +1143,22 @@ void qlifealgo::step() {
       if (poller->isInterrupted())
          break ;
       t -= 1 ;
-      if (t > stepsize) // might change; make it happen now
-         t = stepsize ;
+      if (t > increment) // might change; make it happen now
+         t = increment ;
    }
 }
 /**
  *   If we change the rule we need to mark everything dirty.
  */
 const char *qlifealgo::setrule(const char *s) {
+   
+   const char* err = global_liferules.setrule(s, this);
+   if (err) return err;
+   if (serial == global_liferules.getSerial())
+      return 0 ;
+   serial = global_liferules.getSerial() ;
    markglobalchange() ;
-   const char *p = global_liferules.setrule(s) ;
-   // AKT: hasB0notS8 flag has been set
+
    if (!global_liferules.hasB0notS8) {
       // current rule doesn't have B0, or it has both B0 and S8
       ruletable = global_liferules.rule0 ;
@@ -1158,7 +1186,36 @@ const char *qlifealgo::setrule(const char *s) {
    } else {
       // ruletable will be set in step() depending on gen parity
    }
-   return p;
+   
+   if (global_liferules.isHexagonal())
+      grid_type = HEX_GRID;
+   else
+      grid_type = SQUARE_GRID;
+
+   // AKT: if the grid is bounded then call uproot() if necessary so that
+   // dogen() never needs to call it
+   if (gridwd > 0 && gridht > 0) {
+      // use the top left and bottom right corners of the grid, but expanded by 2
+      // to allow for growth in the borders when the grid edges are joined
+      int xmin = -int(gridwd/2) - 2;
+      int ymin = -int(gridht/2) - 2;
+      int xmax = xmin + gridwd + 3;
+      int ymax = ymin + gridht + 3;
+      // duplicate the expansion code in setcell()
+      ymin = -ymin;
+      ymax = -ymax;
+      if (generation.odd()) {
+         xmin--;
+         ymin--;
+         xmax--;
+         ymax--;
+      }
+      // min is -ve, max is +ve, xmin is -ve, xmax is +ve, ymin is +ve, ymax is -ve
+      while (xmin < min || xmax > max || ymin > max || ymax < min)
+         uproot();
+   }
+   
+   return 0;
 }
 
 static lifealgo *creator() { return new qlifealgo() ; }
