@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -220,12 +220,10 @@ public:
      // poller->bailIfCalculating() ;
      popValid = 0 ;
    }
-   virtual void setIncrement(bigint inc) { stepsize = inc ; }
-   virtual void setIncrement(int inc) { stepsize = inc ; }
+   virtual void setIncrement(bigint inc) { increment = inc ; }
+   virtual void setIncrement(int inc) { increment = inc ; }
    virtual void setGeneration(bigint gen) { generation = gen ; }
-   virtual const bigint &getIncrement() { return stepsize ; }
    virtual const bigint &getPopulation() ;
-   virtual const bigint &getGeneration() { return generation ; }
    virtual int isEmpty() ;
    // can we do the gen count doubling? only hashlife
    virtual int hyperCapable() { return 0 ; }
@@ -234,6 +232,8 @@ public:
    virtual const char *setrule(const char *s) ;
    virtual const char *getrule() { return global_liferules.getrule() ; }
    virtual void step() ;
+   virtual void* getcurrentstate() { return 0 ; }
+   virtual void setcurrentstate(void *) {}
    virtual void draw(viewport &view, liferender &renderer) ;
    virtual void fit(viewport &view, int force) ;
    virtual void lowerRightPixel(bigint &x, bigint &y, int mag) ;
@@ -273,14 +273,12 @@ private:
    void allsub(vector<supertile*> &src, vector<supertile*> &dst, int lev) ;
    int gethbitsfromleaves(vector<supertile *> v) ;
    int getvbitsfromleaves(vector<supertile *> v) ;
-   void markglobalchange(supertile *, int) ;
+   int markglobalchange(supertile *, int) ;
    void markglobalchange() ; // call if the rule changes
    /* data elements */
    int min, max, rootlev ;
    int minlow32 ;
    bigint bmin, bmax ;
-   bigint generation ;
-   bigint stepsize ;
    bigint population ;
    int popValid ;
    linkedmem *tilelist, *supertilelist, *bricklist ;
@@ -301,5 +299,6 @@ private:
    int quickb, deltaforward ;
    int llbits, llsize ;
    char *llxb, *llyb ;
+   int serial ; // last serial value from global_rules
 } ;
 #endif
