@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -23,7 +23,8 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
                         / ***/
 #include "readpattern.h"
 #include "lifealgo.h"
-#include "util.h"          // for *progress calls
+#include "liferules.h"     // for MAXRULESIZE
+#include "util.h"          // for lifewarning and *progress calls
 #include <cstdio>
 #ifdef ZLIB
 #include <zlib.h>
@@ -141,7 +142,7 @@ const char *readtextpattern(lifealgo &imp, char *line) {
 /*
  *   Parse "#CXRLE key=value key=value ..." line and extract values.
  */
-void ParseXRLELine(char *line, int *xoff, int *yoff, bigint &gen) {
+void ParseXRLELine(char *line, int *xoff, int *yoff, bool *sawpos, bigint &gen) {
    char *key = line;
    while (key) {
       // set key to start of next key word
@@ -158,6 +159,7 @@ void ParseXRLELine(char *line, int *xoff, int *yoff, bigint &gen) {
       if (strncmp(key, "Pos", 3) == 0) {
          // extract Pos=int,int
          sscanf(value, "%d,%d", xoff, yoff);
+         *sawpos = true;
 
       } else if (strncmp(key, "Gen", 3) == 0) {
          // extract Gen=bigint
@@ -184,14 +186,13 @@ const char *readrle(lifealgo &imp, char *line) {
    const char *errmsg;
    int wd=0, ht=0, xoff=0, yoff=0;
    bigint gen = bigint::zero;
-   bool xrle = false;               // extended RLE format?
+   bool sawpos = false;             // xoff and yoff set in ParseXRLELine?
    bool sawrule = false;            // saw explicit rule?
 
    // parse any #CXRLE line(s) at start
    while (strncmp(line, "#CXRLE", 6) == 0) {
-      ParseXRLELine(line, &xoff, &yoff, gen);
+      ParseXRLELine(line, &xoff, &yoff, &sawpos, gen);
       imp.setGeneration(gen);
-      xrle = true;
       if (getline(line, LINESIZE) == NULL) return 0;
    }
 
@@ -220,20 +221,6 @@ const char *readrle(lifealgo &imp, char *line) {
          while (*p && *p != '=') p++; p++;
          sscanf(p, "%d", &ht);
 
-         /* we no longer center RLE pattern around 0,0
-         if (!xrle) {
-            xoff = -(wd / 2);
-            yoff = -(ht / 2);
-         }
-         */
-
-         if (getedges) {
-            top = yoff;
-            left = xoff;
-            bottom = yoff + ht - 1;
-            right = xoff + wd - 1;
-         }
-
          while (*p && *p != 'r') p++;
          if (strncmp(p, "rule", 4) == 0) {
             p += 4;
@@ -254,6 +241,20 @@ const char *readrle(lifealgo &imp, char *line) {
             errmsg = imp.setrule("B3/S23");
             if (errmsg) return errmsg;
          }
+
+         // imp.setrule() has set imp.gridwd and imp.gridht
+         if (!sawpos && (imp.gridwd > 0 || imp.gridht > 0)) {
+            // position pattern at top left corner of bounded grid
+            xoff = -int(imp.gridwd / 2);
+            yoff = -int(imp.gridht / 2);
+         }
+
+         if (getedges) {
+            top = yoff;
+            left = xoff;
+            bottom = yoff + ht - 1;
+            right = xoff + wd - 1;
+         }
       } else {
          n = 0 ;
          for (p=line; *p; p++) {
@@ -402,7 +403,9 @@ const char *readdblife(lifealgo &imp, char *line) {
 // See http://psoup.math.wisc.edu/mcell/ca_files_formats.html for details.
 //
 const char *readmcell(lifealgo &imp, char *line) {
-   int x=0, y=0;
+   int x = 0, y = 0;
+   int wd = 0, ht = 0;              // bounded if > 0
+   int wrapped = 0;                 // plane if 0, torus if 1
    char *p;
    const char *errmsg;
    bool sawrule = false;            // saw explicit rule?
@@ -430,7 +433,7 @@ const char *readmcell(lifealgo &imp, char *line) {
                   if (c == '.') {
                      x += n;
                   } else if (c == '$') {
-                     x = 0;
+                     x = -(wd/2);
                      y += n;
                   } else {
                      int state = 0;
@@ -459,6 +462,18 @@ const char *readmcell(lifealgo &imp, char *line) {
                }
             }
 
+         // look for bounded universe
+         } else if (strncmp(line, "#BOARD ", 7) == 0) {
+            sscanf(line + 7, "%dx%d", &wd, &ht);
+            // write pattern in top left corner initially
+            // (pattern will be shifted to middle of grid below)
+            x = -(wd/2);
+            y = -(ht/2);
+
+         // look for topology
+         } else if (strncmp(line, "#WRAP ", 6) == 0) {
+            sscanf(line + 6, "%d", &wrapped);
+
          // we allow lines like "#GOLLY WireWorld"
          } else if (!sawrule && (strncmp(line, "#GOLLY", 6) == 0 ||
                                  strncmp(line, "#RULE", 5) == 0) ) {
@@ -493,6 +508,42 @@ const char *readmcell(lifealgo &imp, char *line) {
          }
       }
    }
+   
+   if (wd > 0 || ht > 0) {
+      // grid is bounded, so append suitable suffix to rule
+      char rule[MAXRULESIZE];
+      if (wrapped)
+         sprintf(rule, "%s:T%d,%d", imp.getrule(), wd, ht);
+      else
+         sprintf(rule, "%s:P%d,%d", imp.getrule(), wd, ht);
+      errmsg = imp.setrule(rule);
+      if (errmsg) {
+         // should never happen
+         lifewarning("Bug in readmcell code!");
+         return errmsg;
+      }
+      // shift pattern to middle of bounded grid
+      imp.endofpattern();
+      if (!imp.isEmpty()) {
+         imp.findedges(&top, &left, &bottom, &right);
+         // pattern is currently in top left corner so shift down and right
+         // (note that we add 1 to wd and ht to get same position as MCell)
+         int shiftx = (wd + 1 - (right.toint() - left.toint() + 1)) / 2;
+         int shifty = (ht + 1 - (bottom.toint() - top.toint() + 1)) / 2;
+         if (shiftx > 0 || shifty > 0) {
+            for (y = bottom.toint(); y >= top.toint(); y--) {
+               for (x = right.toint(); x >= left.toint(); x--) {
+                  int state = imp.getcell(x, y);
+                  if (state > 0) {
+                     imp.setcell(x, y, 0);
+                     imp.setcell(x+shiftx, y+shifty, state);
+                  }
+               }
+            }
+         }
+      }
+   }
+   
    return 0;
 }
 
