@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -117,11 +117,6 @@ pair<double, double> viewport::atf(int x, int y) {
  *   generation count).  In the case of mag < 0, it always returns
  *   the upper left pixel; it's up to the caller to adjust when
  *   mag<0.
- *
- *   If the pixel is offscreen, it will return a value that is also
- *   offscreen, but only by one pixel.
- *   AKT: This code has been commented out because it makes it
- *   harder to calculate an accurate paste rectangle.
  */
 pair<int,int> viewport::screenPosOf(bigint x, bigint y, lifealgo *algo) {
    if (mag < 0) {
@@ -140,23 +135,40 @@ pair<int,int> viewport::screenPosOf(bigint x, bigint y, lifealgo *algo) {
    int xx = 0 ;
    int yy = 0 ;
    
-/* AKT: don't do this clipping
+/* AKT: don't do this clipping as it makes it harder to
+        calculate an accurate paste rectangle
    if (x < 0)
      xx = -1 ;
    else if (x > getxmax())
      xx = getxmax() + 1 ;
    else
-*/
      xx = x.toint() ;
      
-/* AKT: don't do this clipping
    if (y < 0)
      yy = -1 ;
    else if (y > getymax())
      yy = getymax() + 1 ;
    else
-*/
      yy = y.toint() ;
+*/
+   
+   // AKT: better to do this in bigint::toint() and make these static consts???
+   bigint bigmaxint(INT_MAX) ;
+   bigint bigminint(INT_MIN) ;
+
+   if (x > bigmaxint)
+      xx = INT_MAX ;
+   else if (x < bigminint)
+      xx = INT_MIN ;
+   else
+      xx = x.toint() ;
+   
+   if (y > bigmaxint)
+      yy = INT_MAX ;
+   else if (y < bigminint)
+      yy = INT_MIN ;
+   else
+      yy = y.toint() ;
      
    return pair<int,int>(xx,yy) ;
 }
