@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -45,12 +45,13 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include "wxrender.h"      // for DrawOneIcon
 #include "wxlayer.h"       // for currlayer, LayerBarHeight, SetLayerColors
 #include "wxundo.h"        // for currlayer->undoredo->...
+#include "wxtimeline.h"    // for TimelineExists
 #include "wxedit.h"
 
 // -----------------------------------------------------------------------------
 
 enum {
-   // ids for bitmap buttons
+   // ids for bitmap buttons in edit bar
    UNDO_BUTT = 0,
    REDO_BUTT,
    DRAW_BUTT,
@@ -61,7 +62,7 @@ enum {
    ZOOMOUT_BUTT,
    ALLSTATES_BUTT,
    NUM_BUTTONS,      // must be after all buttons
-   SCROLL_BAR
+   STATE_BAR
 };
 
 #ifdef __WXMSW__
@@ -173,7 +174,7 @@ BEGIN_EVENT_TABLE(EditBar, wxPanel)
    EVT_LEFT_DOWN        (              EditBar::OnMouseDown)
    EVT_LEFT_DCLICK      (              EditBar::OnMouseDown)
    EVT_BUTTON           (wxID_ANY,     EditBar::OnButton)
-   EVT_COMMAND_SCROLL   (SCROLL_BAR,   EditBar::OnScroll)
+   EVT_COMMAND_SCROLL   (STATE_BAR,    EditBar::OnScroll)
 END_EVENT_TABLE()
 
 // -----------------------------------------------------------------------------
@@ -193,7 +194,7 @@ const int BOXSIZE = 17;             // width and height of colorbox and iconbox
 const int BOXGAP = 8;               // gap between colorbox and iconbox
 const int PAGESIZE = 10;            // scroll amount when paging
 
-// edit bar buttons (must be global to use Connect/Disconect on Windows)
+// edit bar buttons (must be global to use Connect/Disconnect on Windows)
 wxBitmapButton* ebbutt[NUM_BUTTONS];
 
 // -----------------------------------------------------------------------------
@@ -318,7 +319,7 @@ EditBar::EditBar(wxWindow* parent, wxCoord xorg, wxCoord yorg, int wd, int ht)
    #endif
    int x = xpos + 3*digitwd + BOXGAP + 2*(BOXSIZE + BOXGAP);
    int y = (SMALLHT - (scrollbarht + 1)) / 2;
-   drawbar = new wxScrollBar(this, SCROLL_BAR, wxPoint(x, y),
+   drawbar = new wxScrollBar(this, STATE_BAR, wxPoint(x, y),
                              wxSize(scrollbarwd, scrollbarht),
                              wxSB_HORIZONTAL);
    if (drawbar == NULL) Fatal(_("Failed to create scroll bar!"));
@@ -340,7 +341,7 @@ void EditBar::SetEditFont(wxDC& dc)
 {
    dc.SetFont(*editfont);
    dc.SetTextForeground(*wxBLACK);
-   dc.SetBrush(*wxBLACK_BRUSH);           // avoids problem on Linux/X11
+   dc.SetBrush(*wxBLACK_BRUSH);
    dc.SetBackgroundMode(wxTRANSPARENT);
 }
 
@@ -538,7 +539,7 @@ void EditBar::OnPaint(wxPaintEvent& WXUNUSED(event))
 {
    int wd, ht;
    GetClientSize(&wd, &ht);
-   // wd or ht might be < 1 on Win/X11 platforms
+   // wd or ht might be < 1 on Windows
    if (wd < 1) wd = 1;
    if (ht < 1) ht = 1;
 
@@ -558,9 +559,7 @@ void EditBar::OnPaint(wxPaintEvent& WXUNUSED(event))
       wxBufferedPaintDC dc(this, *editbitmap);
    #endif
    
-   if (!showedit) return;
-   
-   DrawEditBar(dc, wd, ht);
+   if (showedit) DrawEditBar(dc, wd, ht);
 }
 
 // -----------------------------------------------------------------------------
@@ -815,10 +814,6 @@ void EditBar::SelectButton(int id, bool select)
       ebbutt[id]->SetBitmapLabel(normbutt[id]);
    }
 
-   #ifdef __WXX11__
-      ebbutt[id]->ClearBackground();    // fix wxX11 problem
-   #endif
-
    ebbutt[id]->Refresh(false);
 }
 
@@ -866,6 +861,7 @@ void UpdateEditBar(bool active)
 {
    if (editbarptr && showedit) {
       if (viewptr->waitingforclick) active = false;
+      bool timeline = TimelineExists();
 
       // set state of toggle buttons
       editbarptr->SelectButton(DRAW_BUTT,       currlayer->curs == curs_pencil);
@@ -881,8 +877,9 @@ void UpdateEditBar(bool active)
       // appear to be active
       bool canundo = (allowundo && (viewptr->drawingcells || viewptr->selectingcells))
                      || currlayer->undoredo->CanUndo();
-      editbarptr->EnableButton(UNDO_BUTT,       active && canundo);
-      editbarptr->EnableButton(REDO_BUTT,       active && currlayer->undoredo->CanRedo());
+      editbarptr->EnableButton(UNDO_BUTT,       active && !timeline && canundo);
+      editbarptr->EnableButton(REDO_BUTT,       active && !timeline &&
+                                                          currlayer->undoredo->CanRedo());
       editbarptr->EnableButton(DRAW_BUTT,       active);
       editbarptr->EnableButton(PICK_BUTT,       active);
       editbarptr->EnableButton(SELECT_BUTT,     active);
@@ -958,3 +955,30 @@ void ShiftEditBar(int yamount)
    editbarptr->GetPosition(&x, &y);
    editbarptr->Move(x, y + yamount);
 }
+
+// -----------------------------------------------------------------------------
+
+void CycleDrawingState(bool higher)
+{
+   if (viewptr->drawingcells) return;
+   
+   int maxstate = currlayer->algo->NumCellStates() - 1;
+   
+   if (higher) {
+      if (currlayer->drawingstate == maxstate)
+         currlayer->drawingstate = 0;
+      else
+         currlayer->drawingstate++;
+   } else {
+      if (currlayer->drawingstate == 0)
+         currlayer->drawingstate = maxstate;
+      else
+         currlayer->drawingstate--;
+   }
+   
+   if (showedit) {
+      editbarptr->Refresh(false);
+      editbarptr->Update();
+      editbarptr->UpdateScrollBar();
+   }
+}
