@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -48,4 +48,7 @@ void ToggleAllStates();
 void ShiftEditBar(int yamount);
 // Shift edit bar up/down by given amount.
 
+void CycleDrawingState(bool higher);
+// Cycle current drawing state to next higher/lower state.
+
 #endif
