@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -56,6 +56,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include "wxalgos.h"       // for CreateNewUniverse, algo_type, algoinfo, etc
 #include "wxlayer.h"       // for currlayer, etc
 #include "wxhelp.h"        // for ShowHelp
+#include "wxtimeline.h"    // for InitTimelineFrame, ToggleTimelineBar, etc
 
 #ifdef __WXMAC__
    #include <Carbon/Carbon.h>                      // for OpaqueWindowPtr, etc
@@ -226,60 +227,67 @@ void MainFrame::NewPattern(const wxString& title)
 
 // -----------------------------------------------------------------------------
 
-bool MainFrame::LoadImage(const wxString& path)
+static bool IsImageFile(const wxString& path)
 {
    wxString ext = path.AfterLast('.');
    // if path has no extension then ext == path
    if (ext == path) return false;
-   
+
+   // supported extensions match image handlers added in GollyApp::OnInit()
+   return   ext.IsSameAs(wxT("bmp"),false) ||
+            ext.IsSameAs(wxT("gif"),false) ||
+            ext.IsSameAs(wxT("png"),false) ||
+            ext.IsSameAs(wxT("tif"),false) ||
+            ext.IsSameAs(wxT("tiff"),false) ||
+            ext.IsSameAs(wxT("icons"),false) ||
+            // we don't actually support JPEG files but let LoadImage handle them
+            ext.IsSameAs(wxT("jpg"),false) ||
+            ext.IsSameAs(wxT("jpeg"),false);
+}
+
+// -----------------------------------------------------------------------------
+
+bool MainFrame::LoadImage(const wxString& path)
+{
    // don't try to load JPEG file
+   wxString ext = path.AfterLast('.');
    if ( ext.IsSameAs(wxT("jpg"),false) ||
         ext.IsSameAs(wxT("jpeg"),false) ) {
       Warning(_("Golly cannot import JPEG data, only BMP/GIF/PNG/TIFF."));
-      // return true so pattern will be empty
+      // pattern will be empty
       return true;
    }
 
-   // supported extensions match image handlers added in GollyApp::OnInit()
-   if ( ext.IsSameAs(wxT("bmp"),false) ||
-        ext.IsSameAs(wxT("gif"),false) ||
-        ext.IsSameAs(wxT("png"),false) ||
-        ext.IsSameAs(wxT("tif"),false) ||
-        ext.IsSameAs(wxT("tiff"),false) ||
-        ext.IsSameAs(wxT("icons"),false) ) {
-      wxImage image;
-      if ( image.LoadFile(path) ) {
-         // don't change the current rule here -- that way the image can
-         // be loaded into any algo
-         unsigned char maskr, maskg, maskb;
-         bool hasmask = image.GetOrFindMaskColour(&maskr, &maskg, &maskb);
-         int wd = image.GetWidth();
-         int ht = image.GetHeight();
-         unsigned char* idata = image.GetData();
-         int x, y;
-         lifealgo* curralgo = currlayer->algo;
-         for (y = 0; y < ht; y++) {
-            for (x = 0; x < wd; x++) {
-               long pos = (y * wd + x) * 3;
-               unsigned char r = idata[pos];
-               unsigned char g = idata[pos+1];
-               unsigned char b = idata[pos+2];
-               if ( hasmask && r == maskr && g == maskg && b == maskb ) {
-                  // treat transparent pixel as a dead cell
-               } else if ( r < 255 || g < 255 || b < 255 ) {
-                  // treat non-white pixel as a live cell
-                  curralgo->setcell(x, y, 1);
-               }
+   wxImage image;
+   if ( image.LoadFile(path) ) {
+      // don't change the current rule here -- that way the image can
+      // be loaded into any algo
+      unsigned char maskr, maskg, maskb;
+      bool hasmask = image.GetOrFindMaskColour(&maskr, &maskg, &maskb);
+      int wd = image.GetWidth();
+      int ht = image.GetHeight();
+      unsigned char* idata = image.GetData();
+      int x, y;
+      lifealgo* curralgo = currlayer->algo;
+      for (y = 0; y < ht; y++) {
+         for (x = 0; x < wd; x++) {
+            long pos = (y * wd + x) * 3;
+            unsigned char r = idata[pos];
+            unsigned char g = idata[pos+1];
+            unsigned char b = idata[pos+2];
+            if ( hasmask && r == maskr && g == maskg && b == maskb ) {
+               // treat transparent pixel as a dead cell
+            } else if ( r < 255 || g < 255 || b < 255 ) {
+               // treat non-white pixel as a live cell
+               curralgo->setcell(x, y, 1);
             }
          }
-         curralgo->endofpattern();
-      } else {
-         Warning(_("Could not load image from file!"));
       }
-      return true;
+      curralgo->endofpattern();
    } else {
-      return false;
+      Warning(_("Could not load image from file!"));
    }
+   return true;
 }
 
 // -----------------------------------------------------------------------------
@@ -292,12 +300,13 @@ void MainFrame::LoadPattern(const wxString& path, const wxString& newtitle,
       return;
    }
 
-   // newtitle is empty if called from ResetPattern/RestorePattern
+   // newtitle is only empty if called from ResetPattern/RestorePattern
    if (!newtitle.IsEmpty()) {
       if (askonload && !inscript && currlayer->dirty && !SaveCurrentLayer()) return;
 
       if (inscript) stop_after_script = true;
       currlayer->savestart = false;
+      currlayer->currfile = path;
       
       // reset step size now in case UpdateStatus is called below
       currlayer->currbase = algoinfo[currlayer->algtype]->defbase;
@@ -336,16 +345,16 @@ void MainFrame::LoadPattern(const wxString& path, const wxString& newtitle,
    delete currlayer->algo;
    currlayer->algo = CreateNewUniverse(currlayer->algtype);
 
-   // ensure new universe uses same rule in case LoadImage succeeds
-   currlayer->algo->setrule( oldrule.mb_str(wxConvLocal) );
-
    if (!newtitle.IsEmpty()) {
       // show new file name in window title but no rule (which readpattern can change);
       // nicer if user can see file name while loading a very large pattern
       MySetTitle(_("Loading ") + newtitle);
    }
 
-   if (LoadImage(path)) {
+   if (IsImageFile(path)) {
+      // ensure new universe uses same rule
+      currlayer->algo->setrule( oldrule.mb_str(wxConvLocal) );
+      LoadImage(path);
       viewptr->nopattupdate = false;
    } else {
       const char* err = readpattern(FILEPATH, *currlayer->algo);
@@ -379,10 +388,20 @@ void MainFrame::LoadPattern(const wxString& path, const wxString& newtitle,
 
    if (!newtitle.IsEmpty()) {
       MarkLayerClean(newtitle);     // calls SetWindowTitle
-   
-      // restore default base step for current algo
-      // (currlayer->currexpo was set to 0 above)
-      currlayer->currbase = algoinfo[currlayer->algtype]->defbase;
+      
+      if (TimelineExists()) {
+         // we've loaded a .mc file with a timeline so go to 1st frame
+         InitTimelineFrame();
+         if (!showtimeline) ToggleTimelineBar();
+         // switch to the base step and exponent used to record the timeline
+         pair<int, int> be = currlayer->algo->getbaseexpo();
+         currlayer->currbase = be.first;
+         currlayer->currexpo = be.second;
+      } else {
+         // restore default base step for current algo
+         // (currlayer->currexpo was set to 0 above)
+         currlayer->currbase = algoinfo[currlayer->algtype]->defbase;
+      }
       SetGenIncrement();
    
       // restore default colors for current algo/rule
@@ -466,9 +485,9 @@ bool MainFrame::ExtractZipEntry(const wxString& zippath,
             // read and write in chunks so we can show a progress dialog
             const int BUFFER_SIZE = 4000;
             char buf[BUFFER_SIZE];
-            int incount = 0;
-            int outcount = 0;
-            int lastread, lastwrite;
+            size_t incount = 0;
+            size_t outcount = 0;
+            size_t lastread, lastwrite;
             double filesize = (double) entry->GetSize();
             if (filesize <= 0.0) filesize = -1.0;        // show indeterminate progress
             
@@ -486,7 +505,7 @@ bool MainFrame::ExtractZipEntry(const wxString& zippath,
                   break;
                }
                char msg[128];
-               sprintf(msg, "File size: %.2g MB", double(incount) / 1048576.0);
+               sprintf(msg, "File size: %.2f MB", double(incount) / 1048576.0);
                if (AbortProgress((double)incount / filesize, wxString(msg,wxConvLocal))) {
                   outcount = 0;
                   break;
@@ -659,9 +678,9 @@ void MainFrame::OpenZipFile(const wxString& zippath)
    }  // end while
 
    if (rulefiles > 0) {
-      contents += wxT("<p>Files marked as \"[installed]\" have been installed into your rules folder<br>\n(");
+      contents += wxT("<p>Files marked as \"[installed]\" have been stored in your rules folder:<br>\n");
       contents += userrules;
-      contents += wxT(").\n");
+      contents += wxT("\n");
    }
    contents += wxT("\n</body></html>");
    
@@ -691,8 +710,7 @@ void MainFrame::OpenZipFile(const wxString& zippath)
             Raise();
             // don't call AddRecentPattern(tempfile) here; OpenFile has added
             // zippath to recent patterns
-            currlayer->currfile = tempfile;
-            LoadPattern(currlayer->currfile, GetBaseName(tempfile), true, scriptfiles == 0);
+            LoadPattern(tempfile, GetBaseName(tempfile), true, scriptfiles == 0);
          }
       }
       if (scriptfiles == 1) {
@@ -752,8 +770,14 @@ void MainFrame::OpenFile(const wxString& path, bool remember)
    } else {
       // load pattern
       if (remember) AddRecentPattern(path);
-      currlayer->currfile = path;
-      LoadPattern(currlayer->currfile, GetBaseName(path));
+      
+      // ensure path is a full path because a script might want to reset() to it
+      // (in which case the cwd is the script's directory, not gollydir)
+      wxString newpath = path;
+      wxFileName fname(newpath);
+      if (!fname.IsAbsolute()) newpath = gollydir + path;
+      
+      LoadPattern(newpath, GetBaseName(path));
    }
 }
 
@@ -966,32 +990,16 @@ void MainFrame::OpenScript()
 bool MainFrame::CopyTextToClipboard(const wxString& text)
 {
    bool result = true;
-   #ifdef __WXX11__
-      // no global clipboard support on X11 so we save data in a file
-      wxFile tmpfile(clipfile, wxFile::write);
-      if ( tmpfile.IsOpened() ) {
-         size_t textlen = text.Length();
-         if ( tmpfile.Write( text.c_str(), textlen ) < textlen ) {
-            Warning(_("Could not write all data to clipboard file!"));
-            result = false;
-         }
-         tmpfile.Close();
-      } else {
-         Warning(_("Could not create clipboard file!"));
+   if (wxTheClipboard->Open()) {
+      if ( !wxTheClipboard->SetData(new wxTextDataObject(text)) ) {
+         Warning(_("Could not copy text to clipboard!"));
          result = false;
       }
-   #else
-      if (wxTheClipboard->Open()) {
-         if ( !wxTheClipboard->SetData(new wxTextDataObject(text)) ) {
-            Warning(_("Could not copy text to clipboard!"));
-            result = false;
-         }
-         wxTheClipboard->Close();
-      } else {
-         Warning(_("Could not open clipboard!"));
-         result = false;
-      }
-   #endif
+      wxTheClipboard->Close();
+   } else {
+      Warning(_("Could not open clipboard!"));
+      result = false;
+   }
    return result;
 }
 
@@ -1049,12 +1057,7 @@ bool MainFrame::GetTextFromClipboard(wxTextDataObject* textdata)
          }
 
       } else {
-         #ifdef __WXX11__
-            statusptr->ErrorMessage(_("Sorry, but there is no clipboard support for X11."));
-            // do X11 apps like xlife or fontforge have clipboard support???
-         #else
-            statusptr->ErrorMessage(_("No data in clipboard."));
-         #endif
+         statusptr->ErrorMessage(_("No data in clipboard."));
       }
       wxTheClipboard->Close();
 
@@ -1078,33 +1081,21 @@ void MainFrame::OpenClipboard()
    }
 
    // load and view pattern data stored in clipboard
-   #ifdef __WXX11__
-      // on X11 the clipboard data is in non-temporary clipfile, so copy
-      // clipfile to tempstart (for use by ResetPattern and ShowPatternInfo)
-      if ( wxCopyFile(clipfile, currlayer->tempstart, true) ) {
-         currlayer->currfile = currlayer->tempstart;
-         LoadPattern(currlayer->currfile, _("clipboard"));
+   wxTextDataObject data;
+   if (GetTextFromClipboard(&data)) {
+      // copy clipboard data to tempstart so we can handle all formats
+      // supported by readpattern
+      wxFile outfile(currlayer->tempstart, wxFile::write);
+      if ( outfile.IsOpened() ) {
+         outfile.Write( data.GetText() );
+         outfile.Close();
+         LoadPattern(currlayer->tempstart, _("clipboard"));
+         // do NOT delete tempstart -- it can be reloaded by ResetPattern
+         // or used by ShowPatternInfo
       } else {
-         statusptr->ErrorMessage(_("Could not copy clipfile!"));
+         statusptr->ErrorMessage(_("Could not create tempstart file!"));
       }
-   #else
-      wxTextDataObject data;
-      if (GetTextFromClipboard(&data)) {
-         // copy clipboard data to tempstart so we can handle all formats
-         // supported by readpattern
-         wxFile outfile(currlayer->tempstart, wxFile::write);
-         if ( outfile.IsOpened() ) {
-            outfile.Write( data.GetText() );
-            outfile.Close();
-            currlayer->currfile = currlayer->tempstart;
-            LoadPattern(currlayer->currfile, _("clipboard"));
-            // do NOT delete tempstart -- it can be reloaded by ResetPattern
-            // or used by ShowPatternInfo
-         } else {
-            statusptr->ErrorMessage(_("Could not create tempstart file!"));
-         }
-      }
-   #endif
+   }
 }
 
 // -----------------------------------------------------------------------------
@@ -1256,8 +1247,7 @@ void MainFrame::OpenRecentPattern(int id)
       OpenFile(path);
       /*
       AddRecentPattern(path);
-      currlayer->currfile = path;
-      LoadPattern(currlayer->currfile, GetBaseName(path));
+      LoadPattern(path, GetBaseName(path));
       */
    }
 }
@@ -1419,6 +1409,10 @@ const char* MainFrame::WritePattern(const wxString& path,
                                     pattern_format format,
                                     int top, int left, int bottom, int right)
 {
+   // if the format is RLE_format and the grid is bounded then force XRLE_format so that
+   // position info is recorded (this position will be used when the file is read)
+   if (format == RLE_format && (currlayer->algo->gridwd > 0 || currlayer->algo->gridht > 0))
+      format = XRLE_format;
    const char* err = writepattern(FILEPATH, *currlayer->algo, format,
                                   top, left, bottom, right);
 
@@ -1442,14 +1436,14 @@ const char* MainFrame::WritePattern(const wxString& path,
 
 // -----------------------------------------------------------------------------
 
-void MainFrame::SavePattern()
+bool MainFrame::SavePattern()
 {
    if (generating) {
       // terminate generating loop and set command_pending flag
       Stop();
       command_pending = true;
       cmdevent.SetId(wxID_SAVE);
-      return;
+      return false;
    }
 
    wxString filetypes;
@@ -1490,7 +1484,7 @@ void MainFrame::SavePattern()
       // allow saving file only if pattern is small enough
       if ( viewptr->OutsideLimits(top, left, bottom, right) ) {
          statusptr->ErrorMessage(_("Pattern is outside +/- 10^9 boundary."));
-         return;
+         return false;
       }
       itop = top.toint();
       ileft = left.toint();
@@ -1536,7 +1530,7 @@ void MainFrame::SavePattern()
          format = MC_format;
       } else {
          statusptr->ErrorMessage(_("Bug in SavePattern!"));
-         return;
+         return false;
       }
 
       const char* err = WritePattern(savedlg.GetPath(), format,
@@ -1547,8 +1541,10 @@ void MainFrame::SavePattern()
          statusptr->DisplayMessage(_("Pattern saved in file: ") + savedlg.GetPath());
          AddRecentPattern(savedlg.GetPath());
          SaveSucceeded(savedlg.GetPath());
+         return true;
       }
    }
+   return false;
 }
 
 // -----------------------------------------------------------------------------
@@ -1821,7 +1817,7 @@ void MainFrame::ShowPrefsDialog(const wxString& page)
       if (tilelayers && numlayers > 1 && tileborder != oldtileborder) {
          int wd, ht;
          bigview->GetClientSize(&wd, &ht);
-         // wd or ht might be < 1 on Win/X11 platforms
+         // wd or ht might be < 1 on Windows
          if (wd < 1) wd = 1;
          if (ht < 1) ht = 1;
          ResizeLayers(wd, ht);
