@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -211,6 +211,12 @@ void SetAppDirectory(const char* argv0)
       // wxMac has set current directory to location of .app bundle so no need
       // to do anything
    #else // assume Unix
+      // first, try to switch to GOLLYDIR if that is set to a sensible value:
+      static const char *gd = STRINGIFY(GOLLYDIR);
+      if ( *gd == '/' && wxSetWorkingDirectory(wxString(gd,wxConvLocal)) ) {
+         return;
+      }
+      // otherwise, use the executable directory as the application directory.
       // user might have started app from a different directory so find
       // last "/" in argv0 and change cwd if "/" isn't part of "./" prefix
       unsigned int pos = strlen(argv0);
@@ -295,11 +301,20 @@ bool GollyApp::OnInit()
    // let non-wx modules call Fatal, Warning, BeginProgress, etc
    lifeerrors::seterrorhandler(&wxerrhandler);
 
-   // allow our .html files to include common graphic formats;
-   // note that wxBMPHandler is always installed
+   // allow .html files to include common graphic formats,
+   // and .icons files to be in any of these formats;
+   // note that wxBMPHandler is always installed, so it needs not be added,
+   // and we can assume that if HAVE_WX_BMP_HANDLER is not defined, then
+   // the handlers have not been auto-detected (and we just install them all).
+#if !defined(HAVE_WX_BMP_HANDLER) || defined(HAVE_WX_GIF_HANDLER)
    wxImage::AddHandler(new wxGIFHandler);
+#endif
+#if !defined(HAVE_WX_BMP_HANDLER) || defined(HAVE_WX_PNG_HANDLER)
    wxImage::AddHandler(new wxPNGHandler);
+#endif
+#if !defined(HAVE_WX_BMP_HANDLER) || defined(HAVE_WX_TIFF_HANDLER)
    wxImage::AddHandler(new wxTIFFHandler);
+#endif
 
    // wxInternetFSHandler is needed to allow downloading files
    wxFileSystem::AddHandler(new wxInternetFSHandler);
@@ -317,16 +332,16 @@ bool GollyApp::OnInit()
    mainptr->SetMinimumStepExponent();
 
    wxString banner = _("This is Golly version ");
-   banner +=         _(STRINGIFY(VERSION)); 
-   banner +=         _(".  Copyright 2009 The Golly Gang.");
+   banner +=         _(STRINGIFY(VERSION));
+   banner +=         _(".  Copyright 2010 The Golly Gang.");
    statusptr->SetMessage(banner);
 
    mainptr->NewPattern();
 
    // script/pattern files are stored in the pendingfiles array for later processing
-   // in OnIdle; this avoids a crash in Win/X11 app if a script is run before
-   // showing the main window, and also avoids event problems in Win app with
-   // a long-running script (eg. user can't hit escape to abort script)
+   // in OnIdle; this avoids a crash in Win app if a script is run before showing
+   // the main window, and also avoids event problems in Win app with a long-running
+   // script (eg. user can't hit escape to abort script)
    const wxString START_PERL = wxT("golly-start.pl");
    const wxString START_PYTHON = wxT("golly-start.py");
    wxString startscript = gollydir + START_PERL;
