@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -35,8 +35,9 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 #include "lifealgo.h"      // for lifealgo class
 
-#include "wxgolly.h"       // for wxGetApp, mainptr
+#include "wxgolly.h"       // for wxGetApp, mainptr, viewptr
 #include "wxmain.h"        // for mainptr->...
+#include "wxview.h"        // for viewptr->...
 #include "wxutils.h"       // for Warning, BeginProgress, etc
 #include "wxprefs.h"       // for GetShortcutTable, helpfontsize, gollydir, etc
 #include "wxscript.h"      // for inscript
@@ -348,7 +349,7 @@ void HelpFrame::OnBackButton(wxCommandEvent& WXUNUSED(event))
    if ( htmlwin->HistoryBack() ) {
       UpdateHelpButtons();
    } else {
-      wxBell();
+      Beep();
    }
 }
 
@@ -359,7 +360,7 @@ void HelpFrame::OnForwardButton(wxCommandEvent& WXUNUSED(event))
    if ( htmlwin->HistoryForward() ) {
       UpdateHelpButtons();
    } else {
-      wxBell();
+      Beep();
    }
 }
 
@@ -409,6 +410,10 @@ void LoadRule(const wxString& rulestring)
    // load recently installed rule.table/tree/colors/icons file
    wxString oldrule = wxString(currlayer->algo->getrule(),wxConvLocal);
    int oldmaxstate = currlayer->algo->NumCellStates() - 1;
+
+   // selection might change if grid becomes smaller,
+   // so save current selection for RememberRuleChange/RememberAlgoChange
+   viewptr->SaveCurrentSelection();
    
    mainptr->Raise();
    
@@ -433,7 +438,7 @@ void LoadRule(const wxString& rulestring)
             delete tempalgo;
             if (!err) {
                // change the current algorithm and switch to the new rule
-               mainptr->ChangeAlgorithm(i, wxString(rulestring,wxConvLocal));
+               mainptr->ChangeAlgorithm(i, rulestring);
                if (i != currlayer->algtype) {
                   currlayer->algo->setrule( oldrule.mb_str(wxConvLocal) );
                   Warning(_("Algorithm could not be changed (pattern is too big to convert)."));
@@ -456,6 +461,11 @@ void LoadRule(const wxString& rulestring)
    if (oldrule != newrule) {
       // show new rule in main window's title but don't change name
       mainptr->SetWindowTitle(wxEmptyString);
+
+      // if grid is bounded then remove any live cells outside grid edges
+      if (currlayer->algo->gridwd > 0 || currlayer->algo->gridht > 0) {
+         mainptr->ClearOutsideGrid();
+      }
    }
    
    // new table/tree might have changed the number of cell states;
@@ -505,9 +515,9 @@ bool DownloadFile(const wxString& url, const wxString& filepath)
             // read and write in chunks so we can show a progress dialog
             const int BUFFER_SIZE = 4000;             // seems ok (on Mac at least)
             char buf[BUFFER_SIZE];
-            int incount = 0;
-            int outcount = 0;
-            int lastread, lastwrite;
+            size_t incount = 0;
+            size_t outcount = 0;
+            size_t lastread, lastwrite;
             double filesize = (double) instream->GetSize();
             if (filesize <= 0.0) filesize = -1.0;     // show indeterminate progress
             
@@ -525,7 +535,7 @@ bool DownloadFile(const wxString& url, const wxString& filepath)
                   break;
                }
                char msg[128];
-               sprintf(msg, "File size: %.2g MB", double(incount) / 1048576.0);
+               sprintf(msg, "File size: %.2f MB", double(incount) / 1048576.0);
                if (AbortProgress((double)incount / filesize, wxString(msg,wxConvLocal))) {
                   // force false result
                   outcount = 0;
@@ -811,8 +821,7 @@ void LoadLexiconPattern()
       }
       
       // load lexicon pattern into current layer
-      currlayer->currfile = currlayer->tempstart;
-      mainptr->LoadPattern(currlayer->currfile, lexicon_name);
+      mainptr->LoadPattern(currlayer->tempstart, lexicon_name);
    } else {
       Warning(_("Could not create tempstart file!"));
    }
@@ -996,20 +1005,10 @@ void HtmlView::CheckAndLoad(const wxString& filepath)
 {
    if (filepath == SHOW_KEYBOARD_SHORTCUTS) {
       // build HTML string to display current keyboard shortcuts
-      wxString contents =
-      wxT("<html><title>Golly Help: Keyboard Shortcuts</title>")
-      wxT("<body bgcolor=\"#FFFFCE\">")
-      wxT("<p><font size=+1><b>Keyboard shortcuts</b></font>")
-      wxT("<p>Use <a href=\"prefs:keyboard\">Preferences > Keyboard</a>")
-      wxT(" to change the following keyboard shortcuts.")
-      wxT("<p><center>")
-      wxT("<table cellspacing=1 border=2 cols=2 width=\"90%\">")
-      wxT("<tr><td align=center>Key Combination</td><td align=center>Action</td></tr>");
-      contents += GetShortcutTable();
-      contents += wxT("</table></center></body></html>");
+      wxString contents = GetShortcutTable();
       
       // write contents to file and call LoadPage so that back/forwards buttons work
-      wxString htmlfile = tempdir + wxT("keyboard_shortcuts.html");
+      wxString htmlfile = tempdir + SHOW_KEYBOARD_SHORTCUTS;
       wxFile outfile(htmlfile, wxFile::write);
       if (outfile.IsOpened()) {
          outfile.Write(contents);
