@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -64,7 +64,7 @@ int numlayers = 0;               // number of existing layers
 int numclones = 0;               // number of cloned layers
 int currindex = -1;              // index of current layer
 
-Layer* currlayer;                // pointer to current layer
+Layer* currlayer = NULL;         // pointer to current layer
 Layer* layer[MAX_LAYERS];        // array of layers
 
 bool cloneavail[MAX_LAYERS];     // for setting unique cloneid
@@ -174,7 +174,7 @@ END_EVENT_TABLE()
 
 static LayerBar* layerbarptr = NULL;      // global pointer to layer bar
 
-// layer bar buttons must be global to use Connect/Disconect on Windows;
+// layer bar buttons must be global to use Connect/Disconnect on Windows;
 // note that bitmapbutt[0..MAX_LAYERS-1] are not used, but it simplifies
 // our logic to have those dummy indices
 static wxBitmapButton* bitmapbutt[NUM_BUTTONS] = {NULL};
@@ -391,6 +391,13 @@ void LayerBar::OnButton(wxCommandEvent& event)
             }
          }
    }
+
+   #ifdef __WXMAC__
+      // avoid weird bug on Mac where viewport can lose keyboard focus after
+      // the user hits DELETE_LAYER button *and* the "All controls" option
+      // is ticked in System Prefs > Keyboard & Mouse > Keyboard Shortcuts
+      viewptr->SetFocus();
+   #endif
 }
 
 // -----------------------------------------------------------------------------
@@ -835,7 +842,7 @@ void ResizeTiles(int bigwd, int bight)
    for ( int i = 0; i < numlayers; i++ ) {
       int wd, ht;
       layer[i]->tilewin->GetClientSize(&wd, &ht);
-      // wd or ht might be < 1 on Win/X11 platforms
+      // wd or ht might be < 1 on Windows
       if (wd < 1) wd = 1;
       if (ht < 1) ht = 1;
       layer[i]->view->resize(wd, ht);
@@ -886,7 +893,7 @@ void CreateTiles()
    // init tilerects, tile window sizes and their viewport sizes
    int wd, ht;
    bigview->GetClientSize(&wd, &ht);
-   // wd or ht might be < 1 on Win/X11 platforms
+   // wd or ht might be < 1 on Windows
    if (wd < 1) wd = 1;
    if (ht < 1) ht = 1;
    ResizeTiles(wd, ht);
@@ -911,7 +918,7 @@ void DestroyTiles()
    // resize viewport in each layer to bigview's client area
    int wd, ht;
    bigview->GetClientSize(&wd, &ht);
-   // wd or ht might be < 1 on Win/X11 platforms
+   // wd or ht might be < 1 on Windows
    if (wd < 1) wd = 1;
    if (ht < 1) ht = 1;
    for ( int i = 0; i < numlayers; i++ )
@@ -995,6 +1002,12 @@ void SyncClones()
             // cloneptr->startmag = currlayer->startmag;
             // cloneptr->startbase = currlayer->startbase;
             // cloneptr->startexpo = currlayer->startexpo;
+
+            // sync timeline settings
+            cloneptr->currframe = currlayer->currframe;
+            cloneptr->autoplay = currlayer->autoplay;
+            cloneptr->tlspeed = currlayer->tlspeed;
+            cloneptr->lastframe = currlayer->lastframe;
          }
       }
    }
@@ -1547,8 +1560,13 @@ void MarkLayerClean(const wxString& title)
    // is equivalent to loading a pattern file)
    if (inscript) currlayer->stayclean = true;
    
-   // set currlayer->currname and call UpdateLayerItem(currindex)
-   mainptr->SetWindowTitle(title);
+   if (title.IsEmpty()) {
+      // pass in currname so UpdateLayerItem(currindex) gets called
+      mainptr->SetWindowTitle(currlayer->currname);
+   } else {
+      // set currlayer->currname to title and call UpdateLayerItem(currindex)
+      mainptr->SetWindowTitle(title);
+   }
    
    if (currlayer->cloneid > 0) {
       // synchronize other clones
@@ -1768,7 +1786,7 @@ static FILE* FindColorFile(const wxString& rule, const wxString& dir)
 
 // -----------------------------------------------------------------------------
 
-static void LoadRuleColors(const wxString& rule, int maxstate)
+static bool LoadRuleColors(const wxString& rule, int maxstate)
 {
    // if rule.colors file exists in userrules or rulesdir then
    // change colors according to info in file
@@ -1812,7 +1830,9 @@ static void LoadRuleColors(const wxString& rule, int maxstate)
          }
       }
       reader.close();
+      return true;
    }
+   return false;
 }
 
 // -----------------------------------------------------------------------------
@@ -1864,6 +1884,69 @@ static bool LoadRuleIcons(const wxString& rule, int maxstate)
 
 // -----------------------------------------------------------------------------
 
+void SetAverageColor(int state, wxBitmap* icon)
+{
+   // set non-icon color to average color of non-black pixels in given icon
+   if (icon) {
+      int wd = icon->GetWidth();
+      int ht = icon->GetHeight();
+      
+      #ifdef __WXMSW__
+         // must use wxNativePixelData for bitmaps with no alpha channel
+         wxNativePixelData icondata(*icon);
+      #else
+         wxAlphaPixelData icondata(*icon);
+      #endif
+
+      if (icondata) {
+         #ifdef __WXMSW__
+            wxNativePixelData::Iterator iconpxl(icondata);
+         #else
+            wxAlphaPixelData::Iterator iconpxl(icondata);
+         #endif
+         
+         int nbcount = 0;  // # of non-black pixels
+         int totalr = 0;
+         int totalg = 0;
+         int totalb = 0;
+
+         for (int i = 0; i < ht; i++) {
+            #ifdef __WXMSW__
+               wxNativePixelData::Iterator iconrow = iconpxl;
+            #else
+               wxAlphaPixelData::Iterator iconrow = iconpxl;
+            #endif
+            for (int j = 0; j < wd; j++) {
+               if (iconpxl.Red() || iconpxl.Green() || iconpxl.Blue()) {
+                  // non-black pixel
+                  totalr += iconpxl.Red();
+                  totalg += iconpxl.Green();
+                  totalb += iconpxl.Blue();
+                  nbcount++;
+               }
+               iconpxl++;
+            }
+            // move to next row of icon bitmap
+            iconpxl = iconrow;
+            iconpxl.OffsetY(icondata, 1);
+         }
+         
+         if (nbcount>0) {
+            currlayer->cellr[state] = int(totalr / nbcount);
+            currlayer->cellg[state] = int(totalg / nbcount);
+            currlayer->cellb[state] = int(totalb / nbcount);
+         }
+         else { // avoid div0
+            currlayer->cellr[state] = 0;
+            currlayer->cellg[state] = 0;
+            currlayer->cellb[state] = 0;
+         }
+      }
+   }
+}
+
+// -----------------------------------------------------------------------------
+
 void UpdateCurrentColors()
 {
    // set current layer's colors and icons according to current algo and rule
@@ -1888,20 +1971,58 @@ void UpdateCurrentColors()
    }
    
    wxString rule = wxString(currlayer->algo->getrule(), wxConvLocal);
-   // replace any '\' and '/' and ':' chars with underscores;
+   // replace any '\' and '/' chars with underscores;
    // ie. given 12/34/6 we look for 12_34_6.{colors|icons}
    rule.Replace(wxT("\\"), wxT("_"));
    rule.Replace(wxT("/"), wxT("_"));
-   rule.Replace(wxT(":"), wxT("_"));
+   
+   // strip off any suffix like ":T100,200" used to specify a bounded grid
+   if (rule.Find(':') >= 0) rule = rule.BeforeFirst(':');
    
    // if rule.colors file exists then override default colors
-   LoadRuleColors(rule, maxstate);
+   bool loadedcolors = LoadRuleColors(rule, maxstate);
    
    // if rule.icons file exists then use those icons
    if ( !LoadRuleIcons(rule, maxstate) ) {
-      // otherwise copy default icons from current algo
-      currlayer->icons15x15 = CopyIcons(ad->icons15x15, 15, maxstate);
-      currlayer->icons7x7 = CopyIcons(ad->icons7x7, 7, maxstate);
+      if (currlayer->algo->getgridtype() == lifealgo::HEX_GRID) {
+         // use hexagonal icons
+         currlayer->icons15x15 = CopyIcons(hexicons15x15, 15, maxstate);
+         currlayer->icons7x7 = CopyIcons(hexicons7x7, 7, maxstate);
+      } else {
+         // otherwise copy default icons from current algo
+         currlayer->icons15x15 = CopyIcons(ad->icons15x15, 15, maxstate);
+         currlayer->icons7x7 = CopyIcons(ad->icons7x7, 7, maxstate);
+      }
+   }   
+   // if rule.colors file wasn't loaded and icons are multi-color then we
+   // set non-icon colors to the average of the non-black pixels in each icon
+   // (note that we use the 7x7 icons because they are faster to scan)
+   wxBitmap** iconmaps = currlayer->icons7x7;
+   if (!loadedcolors && iconmaps && iconmaps[1] && iconmaps[1]->GetDepth() > 1) {
+      for (int n = 1; n <= maxstate; n++) {
+         SetAverageColor(n, iconmaps[n]);
+      }
+      // if extra 15x15 icon was supplied then use it to set state 0 color
+      iconmaps = currlayer->icons15x15;
+      if (iconmaps && iconmaps[0]) {
+         #ifdef __WXMSW__
+            // must use wxNativePixelData for bitmaps with no alpha channel
+            wxNativePixelData icondata(*iconmaps[0]);
+         #else
+            wxAlphaPixelData icondata(*iconmaps[0]);
+         #endif
+         if (icondata) {
+            #ifdef __WXMSW__
+               wxNativePixelData::Iterator iconpxl(icondata);
+            #else
+               wxAlphaPixelData::Iterator iconpxl(icondata);
+            #endif
+            // iconpxl is the top left pixel
+            currlayer->cellr[0] = iconpxl.Red();
+            currlayer->cellg[0] = iconpxl.Green();
+            currlayer->cellb[0] = iconpxl.Blue();
+         }
+      }
    }
    
    if (swapcolors) {
@@ -1993,6 +2114,11 @@ Layer::Layer()
    icons15x15 = NULL;            // no 15x15 icons
    icons7x7 = NULL;              // no 7x7 icons
 
+   currframe = 0;                // first frame in timeline
+   autoplay = 0;                 // not playing
+   tlspeed = 0;                  // default speed for autoplay
+   lastframe = 0;                // no frame displayed
+
    deadbrush = new wxBrush(*wxBLACK);
    gridpen = new wxPen(*wxBLACK);
    boldpen = new wxPen(*wxBLACK);
@@ -2005,8 +2131,13 @@ Layer::Layer()
    if (view == NULL) Fatal(_("Failed to create viewport!"));
 
    if (numlayers == 0) {
-      // creating very first layer
+      // creating very first layer (can't be a clone)
+      cloneid = 0;
       
+      // initialize cloneavail array (cloneavail[0] is never used)
+      cloneavail[0] = false;
+      for (int i = 1; i < MAX_LAYERS; i++) cloneavail[i] = true;
+
       // set some options using initial values stored in prefs file
       algtype = initalgo;
       hyperspeed = inithyperspeed;
@@ -2038,13 +2169,6 @@ Layer::Layer()
       curs = curs_pencil;
       drawingstate = 1;
       
-      // first layer can't be a clone
-      cloneid = 0;
-      
-      // initialize cloneavail array (cloneavail[0] is never used)
-      cloneavail[0] = false;
-      for (int i = 1; i < MAX_LAYERS; i++) cloneavail[i] = true;
-
    } else {
       // adding a new layer after currlayer (see AddLayer)
 
@@ -2074,6 +2198,12 @@ Layer::Layer()
          algo = currlayer->algo;
          undoredo = currlayer->undoredo;
 
+         // clones also share the same timeline
+         currframe = currlayer->currframe;
+         autoplay = currlayer->autoplay;
+         tlspeed = currlayer->tlspeed;
+         lastframe = currlayer->lastframe;
+
          // clones use same name for starting file
          tempstart = currlayer->tempstart;
 
@@ -2294,11 +2424,11 @@ void CellPanel::OnPaint(wxPaintEvent& WXUNUSED(event))
    #endif
 
    // draw cell boxes
+   wxBitmap** iconmaps = currlayer->icons15x15;
    wxRect r = wxRect(0, 0, CELLSIZE+1, CELLSIZE+1);
    int col = 0;
    for (int state = 0; state < 256; state++) {
       if (state < currlayer->algo->NumCellStates()) {
-         wxBitmap** iconmaps = currlayer->icons15x15;
          if (showicons && iconmaps && iconmaps[state]) {
             dc.SetBrush(*wxTRANSPARENT_BRUSH);
             dc.DrawRectangle(r);
