@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -30,6 +30,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include "wx/dataobj.h"    // for wxTextDataObject
 
 #include "bigint.h"        // for bigint
+#include "lifealgo.h"      // for lifealgo
 #include "writepattern.h"  // for pattern_format
 #include "wxprefs.h"       // for MAX_RECENT
 #include "wxalgos.h"       // for MAX_ALGOS, algo_type
@@ -75,7 +76,7 @@ public:
    void ChangeScriptDir();
    void SetPatternDir(const wxString& newdir);
    void SetScriptDir(const wxString& newdir);
-   void SavePattern();
+   bool SavePattern();
    bool SaveCurrentLayer();
    const char* SaveFile(const wxString& path, const wxString& format, bool remember);
    const char* WritePattern(const wxString& path, pattern_format format,
@@ -112,15 +113,20 @@ public:
    void ToggleHashInfo();
    void SetStepExponent(int newexpo);
    void SetGenIncrement();
+   bool SaveStartingPattern();
    void ResetPattern(bool resetundo = true);
    void SetGeneration();
    const char* ChangeGenCount(const char* genstring, bool inundoredo = false);
    void SetBaseStep();
+   void ClearOutsideGrid();
    void ReduceCellStates(int newmaxstate);
    void ShowRuleDialog();
    void ChangeAlgorithm(algo_type newalgotype,
                         const wxString& newrule = wxEmptyString,
                         bool inundoredo = false);
+   bool CreateBorderCells(lifealgo* curralgo);
+   bool DeleteBorderCells(lifealgo* curralgo);
+   void ClearRect(lifealgo* curralgo, int top, int left, int bottom, int right);
 
    // view functions
    void ToggleStatusBar();
@@ -195,7 +201,6 @@ private:
    void OpenZipFile(const wxString& path);
 
    // control functions
-   bool SaveStartingPattern();
    void DisplayPattern();
    bool StepPattern();
 
@@ -302,6 +307,8 @@ enum {
    ID_AUTO,
    ID_HYPER,
    ID_HINFO,
+   ID_RECORD,
+   ID_DELTIME,
    ID_SETALGO,
    ID_SETRULE,
 
@@ -328,6 +335,7 @@ enum {
    ID_ICONS,
    ID_INVERT,
    ID_BUFF,
+   ID_TIMELINE,
    ID_INFO,
 
    // Set Scale submenu
@@ -372,6 +380,7 @@ enum {
    ID_HELP_HELP,
    ID_HELP_REFS,
    ID_HELP_FORMATS,
+   ID_HELP_BOUNDED,
    ID_HELP_PROBLEMS,
    ID_HELP_CHANGES,
    ID_HELP_CREDITS,
