@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -72,6 +72,10 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 // avoid warning about _ being redefined
 #undef _
 
+// on Windows, wxWidgets redefines uid_t/gid_t which breaks Perl's typedefs:
+#undef uid_t
+#undef gid_t
+
 #include <EXTERN.h>
 #include <perl.h>
 #include <XSUB.h>
@@ -96,7 +100,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
    #define PERL589_OR_LATER
 #endif
 
-// check if we're building with Perl 5.10
+// check if we're building with Perl 5.10 or later
 #if (ACTIVEPERL_VERSION >= 1000)
    #define PERL510_OR_LATER
 #endif
@@ -104,6 +108,14 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
    #define PERL510_OR_LATER
 #endif
 
+// check if we're building with Perl 5.10.1 or later
+#if (PERL_REVISION == 5) && (PERL_VERSION == 10) && (PERL_SUBVERSION >= 1)
+   #define PERL5101_OR_LATER
+#endif
+#if (PERL_REVISION == 5) && (PERL_VERSION >= 11)
+   #define PERL5101_OR_LATER
+#endif
+
 // restore wxWidgets definition for _ (from include/wx/intl.h)
 #undef _
 #define _(s) wxGetTranslation(_T(s))
@@ -131,8 +143,9 @@ EXTERN_C void boot_DynaLoader(pTHX_ CV* cv);
 // declare G_* wrappers for the functions we want to use from Perl lib
 extern "C"
 {
+#ifdef USE_ITHREADS
    perl_key*(*G_Perl_Gthr_key_ptr)(register PerlInterpreter*);
-   U8*(*G_Perl_Iexit_flags_ptr)(register PerlInterpreter*);
+#endif
    SV**(*G_Perl_av_fetch)(pTHX_ AV*, I32, I32);
    I32(*G_Perl_av_len)(pTHX_ AV*);
    void(*G_Perl_av_push)(pTHX_ AV*, SV*);
@@ -155,27 +168,43 @@ extern "C"
    int(*G_perl_run)(PerlInterpreter*);
    SV*(*G_Perl_eval_pv)(pTHX_ const char*, I32);
 #ifdef PERL589_OR_LATER
-   IV (*G_Perl_sv_2iv_flags)(pTHX_ SV* sv, I32 flags);
+   IV(*G_Perl_sv_2iv_flags)(pTHX_ SV* sv, I32 flags);
 #endif
 #ifdef PERL510_OR_LATER
    void(*G_Perl_sys_init3)(int*, char***, char***);
    void(*G_Perl_sys_term)(void);
-   I32**(*G_Perl_Imarkstack_ptr_ptr)(register PerlInterpreter*);
-   SV***(*G_Perl_Istack_base_ptr)(register PerlInterpreter*);
-   SV***(*G_Perl_Istack_max_ptr)(register PerlInterpreter*);
-   SV***(*G_Perl_Istack_sp_ptr)(register PerlInterpreter*);
-#else
-   I32**(*G_Perl_Tmarkstack_ptr_ptr)(register PerlInterpreter*);
-   SV***(*G_Perl_Tstack_base_ptr)(register PerlInterpreter*);
-   SV***(*G_Perl_Tstack_max_ptr)(register PerlInterpreter*);
-   SV***(*G_Perl_Tstack_sp_ptr)(register PerlInterpreter*);
+#endif
+#ifdef PERL5101_OR_LATER
+   SV*(*G_Perl_newSV_type)(pTHX_ svtype type);
 #endif
    void(*G_boot_DynaLoader)(pTHX_ CV*);
+
+#ifdef MULTIPLICITY
+   #ifdef PERL510_OR_LATER
+      SV***(*G_Perl_Istack_sp_ptr)(register PerlInterpreter*);
+      SV***(*G_Perl_Istack_base_ptr)(register PerlInterpreter*);
+      SV***(*G_Perl_Istack_max_ptr)(register PerlInterpreter*);
+      I32**(*G_Perl_Imarkstack_ptr_ptr)(register PerlInterpreter*);
+   #else
+      SV***(*G_Perl_Tstack_sp_ptr)(register PerlInterpreter*);
+      SV***(*G_Perl_Tstack_base_ptr)(register PerlInterpreter*);
+      SV***(*G_Perl_Tstack_max_ptr)(register PerlInterpreter*);
+      I32**(*G_Perl_Tmarkstack_ptr_ptr)(register PerlInterpreter*);
+   #endif
+   U8*(*G_Perl_Iexit_flags_ptr)(register PerlInterpreter*);
+   signed char *(*G_Perl_Iperl_destruct_level_ptr)(register PerlInterpreter*);
+#else
+   SV ***G_PL_stack_sp;
+   SV ***G_PL_stack_base;
+   SV ***G_PL_stack_max;
+   I32 **G_PL_markstack_ptr;
+   U8 *G_PL_exit_flags;
+   signed char *G_PL_perl_destruct_level;
+#endif
 }
 
 // redefine Perl functions to their equivalent G_* wrappers
 #define Perl_Gthr_key_ptr        G_Perl_Gthr_key_ptr
-#define Perl_Iexit_flags_ptr     G_Perl_Iexit_flags_ptr
 #define Perl_av_fetch            G_Perl_av_fetch
 #define Perl_av_len              G_Perl_av_len
 #define Perl_av_push             G_Perl_av_push
@@ -203,15 +232,31 @@ extern "C"
 #ifdef PERL510_OR_LATER
    #define Perl_sys_init3           G_Perl_sys_init3
    #define Perl_sys_term            G_Perl_sys_term
-   #define Perl_Imarkstack_ptr_ptr  G_Perl_Imarkstack_ptr_ptr
-   #define Perl_Istack_base_ptr     G_Perl_Istack_base_ptr
-   #define Perl_Istack_max_ptr      G_Perl_Istack_max_ptr
-   #define Perl_Istack_sp_ptr       G_Perl_Istack_sp_ptr
-#else
-   #define Perl_Tmarkstack_ptr_ptr  G_Perl_Tmarkstack_ptr_ptr
-   #define Perl_Tstack_base_ptr     G_Perl_Tstack_base_ptr
-   #define Perl_Tstack_max_ptr      G_Perl_Tstack_max_ptr
-   #define Perl_Tstack_sp_ptr       G_Perl_Tstack_sp_ptr
+#endif
+#ifdef MULTIPLICITY
+   #ifdef PERL510_OR_LATER
+      #define Perl_Imarkstack_ptr_ptr  G_Perl_Imarkstack_ptr_ptr
+      #define Perl_Istack_base_ptr     G_Perl_Istack_base_ptr
+      #define Perl_Istack_max_ptr      G_Perl_Istack_max_ptr
+      #define Perl_Istack_sp_ptr       G_Perl_Istack_sp_ptr
+   #else
+      #define Perl_Tmarkstack_ptr_ptr  G_Perl_Tmarkstack_ptr_ptr
+      #define Perl_Tstack_base_ptr     G_Perl_Tstack_base_ptr
+      #define Perl_Tstack_max_ptr      G_Perl_Tstack_max_ptr
+      #define Perl_Tstack_sp_ptr       G_Perl_Tstack_sp_ptr
+   #endif
+   #define Perl_Iexit_flags_ptr          G_Perl_Iexit_flags_ptr
+   #define Perl_Iperl_destruct_level_ptr G_Perl_Iperl_destruct_level_ptr
+#else  /* no MULTIPLICITY */
+   #define PL_stack_sp               (*G_PL_stack_sp)
+   #define PL_stack_base             (*G_PL_stack_base)
+   #define PL_stack_max              (*G_PL_stack_max)
+   #define PL_markstack_ptr          (*G_PL_markstack_ptr)
+   #define PL_exit_flags             (*G_PL_exit_flags)
+   #define PL_perl_destruct_level    (*G_PL_perl_destruct_level)
+#endif
+#ifdef PERL5101_OR_LATER
+   #define Perl_newSV_type          G_Perl_newSV_type
 #endif
 #define boot_DynaLoader          G_boot_DynaLoader
 
@@ -230,8 +275,9 @@ static struct PerlFunc
    PERL_PROC* ptr;         // function pointer
 } perlFuncs[] =
 {
+#ifdef USE_ITHREADS
    PERL_FUNC(Perl_Gthr_key_ptr)
-   PERL_FUNC(Perl_Iexit_flags_ptr)
+#endif
    PERL_FUNC(Perl_av_fetch)
    PERL_FUNC(Perl_av_len)
    PERL_FUNC(Perl_av_push)
@@ -259,15 +305,33 @@ static struct PerlFunc
 #ifdef PERL510_OR_LATER
    PERL_FUNC(Perl_sys_init3)
    PERL_FUNC(Perl_sys_term)
-   PERL_FUNC(Perl_Imarkstack_ptr_ptr)
-   PERL_FUNC(Perl_Istack_base_ptr)
-   PERL_FUNC(Perl_Istack_max_ptr)
-   PERL_FUNC(Perl_Istack_sp_ptr)
-#else
-   PERL_FUNC(Perl_Tmarkstack_ptr_ptr)
-   PERL_FUNC(Perl_Tstack_base_ptr)
-   PERL_FUNC(Perl_Tstack_max_ptr)
-   PERL_FUNC(Perl_Tstack_sp_ptr)
+#endif
+#ifdef MULTIPLICITY
+   #ifdef PERL510_OR_LATER
+      PERL_FUNC(Perl_Imarkstack_ptr_ptr)
+      PERL_FUNC(Perl_Istack_base_ptr)
+      PERL_FUNC(Perl_Istack_max_ptr)
+      PERL_FUNC(Perl_Istack_sp_ptr)
+   #else
+      PERL_FUNC(Perl_Tmarkstack_ptr_ptr)
+      PERL_FUNC(Perl_Tstack_base_ptr)
+      PERL_FUNC(Perl_Tstack_max_ptr)
+      PERL_FUNC(Perl_Tstack_sp_ptr)
+   #endif
+   PERL_FUNC(Perl_Iexit_flags_ptr)
+   PERL_FUNC(Perl_Iperl_destruct_level_ptr)
+#else  /* no MULTIPLICITY */
+   /* N.B. these are actually variables, not functions, but the distinction does
+      not matter for symbol resolution: */
+   PERL_FUNC(PL_stack_sp)
+   PERL_FUNC(PL_stack_base)
+   PERL_FUNC(PL_stack_max)
+   PERL_FUNC(PL_markstack_ptr)
+   PERL_FUNC(PL_exit_flags)
+   PERL_FUNC(PL_perl_destruct_level)
+#endif
+#ifdef PERL5101_OR_LATER
+   PERL_FUNC(Perl_newSV_type)
 #endif
    PERL_FUNC(boot_DynaLoader)
    { _T(""), NULL }
@@ -292,18 +356,18 @@ static bool LoadPerlLib()
    // don't log errors in here
    wxLogNull noLog;
 
-   // wxDL_GLOBAL corresponds to RTLD_GLOBAL on Linux (ignored on Windows) and
-   // is needed to avoid an ImportError when importing some modules (eg. time)
+   // wxDL_GLOBAL corresponds to RTLD_GLOBAL on Linux (ignored on Windows)
    while ( !dynlib.Load(perllib, wxDL_NOW | wxDL_VERBATIM | wxDL_GLOBAL) ) {
       // prompt user for a different Perl library;
       // on Windows perllib should be something like "perl510.dll"
       // and on Linux it should be something like "libperl.so.5.10"
-      wxBell();
+      Beep();
       wxString str = _("If Perl isn't installed then you'll have to Cancel,");
-      str +=         _("\notherwise change the version numbers and try again.");
+      str +=         _("\notherwise change the version numbers to match the");
+      str +=         _("\nversion installed on your system and try again.");
       #ifdef __WXMSW__
-         str +=      _("\nDepending on where you installed Perl you might have");
-         str +=      _("\nto enter a full path like C:\\Perl\\bin\\perl510.dll.");
+         str +=      _("\n\nIf that fails, search your system for a perl*.dll");
+         str +=      _("\nfile and enter the full path to that file.");
       #endif
       wxTextEntryDialog dialog( wxGetActiveWindow(), str,
                                 _("Could not load the Perl library"),
@@ -354,7 +418,7 @@ static bool LoadPerlLib()
 
 #define RETURN_IF_ABORTED if (PerlScriptAborted()) Perl_croak(aTHX_ NULL)
 
-#define PERL_ERROR(msg) { Perl_croak(aTHX_ msg); }
+#define PERL_ERROR(msg) { Perl_croak(aTHX_ "%s", msg); }
 
 #define CheckRGB(r,g,b,cmd)                                          \
    if (r < 0 || r > 255 || g < 0 || g > 255 || g < 0 || g > 255) {   \
@@ -677,6 +741,9 @@ XS(pl_store)
       int item = ints_per_cell * n;
       int x = SvIV( *av_fetch(inarray, item, 0) );
       int y = SvIV( *av_fetch(inarray, item + 1, 0) );
+      // check if x,y is outside bounded grid
+      const char* err = GSF_checkpos(tempalgo, x, y);
+      if (err) { delete tempalgo; PERL_ERROR(err); }
       if (multistate) {
          int state = SvIV( *av_fetch(inarray, item + 2, 0) );
          if (tempalgo->setcell(x, y, state) < 0) {
@@ -698,8 +765,10 @@ XS(pl_store)
    // write pattern to given file in RLE/XRLE format
    bigint top, left, bottom, right;
    tempalgo->findedges(&top, &left, &bottom, &right);
-   const char* err = writepattern(FILENAME, *tempalgo,
-                        savexrle ? XRLE_format : RLE_format,
+   pattern_format format = savexrle ? XRLE_format : RLE_format;
+   // if grid is bounded then force XRLE_format so that position info is recorded
+   if (tempalgo->gridwd > 0 || tempalgo->gridht > 0) format = XRLE_format;
+   const char* err = writepattern(FILENAME, *tempalgo, format,
                         top.toint(), left.toint(), bottom.toint(), right.toint());
    delete tempalgo;
    if (err) PERL_ERROR(err);
@@ -1158,6 +1227,9 @@ XS(pl_evolve)
       int item = ints_per_cell * n;
       int x = SvIV( *av_fetch(inarray, item, 0) );
       int y = SvIV( *av_fetch(inarray, item + 1, 0) );
+      // check if x,y is outside bounded grid
+      const char* err = GSF_checkpos(tempalgo, x, y);
+      if (err) { delete tempalgo; PERL_ERROR(err); }
       if (multistate) {
          int state = SvIV( *av_fetch(inarray, item + 2, 0) );
          if (tempalgo->setcell(x, y, state) < 0) {
@@ -1178,8 +1250,25 @@ XS(pl_evolve)
 
    // advance pattern by ngens
    mainptr->generating = true;
-   tempalgo->setIncrement(ngens);
-   tempalgo->step();
+   if (tempalgo->gridwd > 0 || tempalgo->gridht > 0) {
+      // a bounded grid must use an increment of 1 so we can call
+      // CreateBorderCells and DeleteBorderCells around each step()
+      tempalgo->setIncrement(1);
+      while (ngens > 0) {
+         if (PerlScriptAborted()) {
+            mainptr->generating = false;
+            delete tempalgo;
+            Perl_croak(aTHX_ NULL);
+         }
+         if (!mainptr->CreateBorderCells(tempalgo)) break;
+         tempalgo->step();
+         if (!mainptr->DeleteBorderCells(tempalgo)) break;
+         ngens--;
+      }
+   } else {
+      tempalgo->setIncrement(ngens);
+      tempalgo->step();
+   }
    mainptr->generating = false;
 
    // convert new pattern into a new cell array
@@ -1269,6 +1358,9 @@ XS(pl_putcells)
             int y = SvIV( *av_fetch(inarray, item + 1, 0) );
             int newx = x0 + x * axx + y * axy;
             int newy = y0 + x * ayx + y * ayy;
+            // check if newx,newy is outside bounded grid
+            err = GSF_checkpos(curralgo, newx, newy);
+            if (err) break;
             int oldstate = curralgo->getcell(newx, newy);
             if (multistate) {
                // multi-state lists can contain dead cells so newstate might be 0
@@ -1291,6 +1383,9 @@ XS(pl_putcells)
          int y = SvIV( *av_fetch(inarray, item + 1, 0) );
          int newx = x0 + x * axx + y * axy;
          int newy = y0 + x * ayx + y * ayy;
+         // check if newx,newy is outside bounded grid
+         err = GSF_checkpos(curralgo, newx, newy);
+         if (err) break;
          int oldstate = curralgo->getcell(newx, newy);
          int newstate;
          if (multistate) {
@@ -1336,6 +1431,9 @@ XS(pl_putcells)
          int y = SvIV( *av_fetch(inarray, item + 1, 0) );
          int newx = x0 + x * axx + y * axy;
          int newy = y0 + x * ayx + y * ayy;
+         // check if newx,newy is outside bounded grid
+         err = GSF_checkpos(curralgo, newx, newy);
+         if (err) break;
          int oldstate = curralgo->getcell(newx, newy);
          if (multistate) {
             // multi-state arrays can contain dead cells so newstate might be 0
@@ -1387,9 +1485,8 @@ XS(pl_getcells)
       int y  = SvIV(ST(1));
       int wd = SvIV(ST(2));
       int ht = SvIV(ST(3));
-      // first check that wd & ht are > 0
-      if (wd <= 0) PERL_ERROR("g_getcells error: width must be > 0.");
-      if (ht <= 0) PERL_ERROR("g_getcells error: height must be > 0.");
+      const char* err = GSF_checkrect(x, y, wd, ht);
+      if (err) PERL_ERROR(err);
       int right = x + wd - 1;
       int bottom = y + ht - 1;
       int cx, cy;
@@ -1514,35 +1611,10 @@ XS(pl_hash)
    int y  = SvIV(ST(1));
    int wd = SvIV(ST(2));
    int ht = SvIV(ST(3));
-   if (wd <= 0) PERL_ERROR("g_hash error: width must be > 0.");
-   if (ht <= 0) PERL_ERROR("g_hash error: height must be > 0.");
-   int right = x + wd - 1;
-   int bottom = y + ht - 1;
-   int cx, cy;
-   int v = 0;
-   int cntr = 0;
+   const char* err = GSF_checkrect(x, y, wd, ht);
+   if (err) PERL_ERROR(err);
 
-   // calculate a hash value for pattern in given rect
-   int hash = 31415962;
-   lifealgo* curralgo = currlayer->algo;
-   for ( cy=y; cy<=bottom; cy++ ) {
-      int yshift = cy - y;
-      for ( cx=x; cx<=right; cx++ ) {
-         int skip = curralgo->nextcell(cx, cy, v);
-         if (skip >= 0) {
-            // found next live cell in this row
-            cx += skip;
-            if (cx <= right) {
-               //note that v is 1 in a two-state universe
-               hash = (hash * 33 + yshift) ^ ((cx - x) * v);
-            }
-         } else {
-            cx = right;  // done this row
-         }
-         cntr++;
-         if ((cntr % 4096) == 0) RETURN_IF_ABORTED;
-      }
-   }
+   int hash = GSF_hash(x, y, wd, ht);
 
    XSRETURN_IV(hash);
 }
@@ -1648,9 +1720,8 @@ XS(pl_select)
       int y  = SvIV(ST(1));
       int wd = SvIV(ST(2));
       int ht = SvIV(ST(3));
-      // first check that wd & ht are > 0
-      if (wd <= 0) PERL_ERROR("g_select error: width must be > 0.");
-      if (ht <= 0) PERL_ERROR("g_select error: height must be > 0.");
+      const char* err = GSF_checkrect(x, y, wd, ht);
+      if (err) PERL_ERROR(err);
       // set selection rect
       GSF_select(x, y, wd, ht);
    }
@@ -1747,7 +1818,14 @@ XS(pl_getcell)
    dXSARGS;
    if (items != 2) PERL_ERROR("Usage: $state = g_getcell($x,$y).");
 
-   int state = currlayer->algo->getcell(SvIV(ST(0)), SvIV(ST(1)));
+   int x = SvIV(ST(0));
+   int y = SvIV(ST(1));
+
+   // check if x,y is outside bounded grid
+   const char* err = GSF_checkpos(currlayer->algo, x, y);
+   if (err) PERL_ERROR(err);
+
+   int state = currlayer->algo->getcell(x, y);
 
    XSRETURN_IV(state);
 }
@@ -2077,6 +2155,30 @@ XS(pl_getrule)
 
 // -----------------------------------------------------------------------------
 
+XS(pl_getwidth)
+{
+   IGNORE_UNUSED_PARAMS;
+   RETURN_IF_ABORTED;
+   dXSARGS;
+   if (items != 0) PERL_ERROR("Usage: $int = g_getwidth().");
+
+   XSRETURN_IV(currlayer->algo->gridwd);
+}
+
+// -----------------------------------------------------------------------------
+
+XS(pl_getheight)
+{
+   IGNORE_UNUSED_PARAMS;
+   RETURN_IF_ABORTED;
+   dXSARGS;
+   if (items != 0) PERL_ERROR("Usage: $int = g_getheight().");
+
+   XSRETURN_IV(currlayer->algo->gridht);
+}
+
+// -----------------------------------------------------------------------------
+
 XS(pl_numstates)
 {
    IGNORE_UNUSED_PARAMS;
@@ -2220,9 +2322,8 @@ XS(pl_visrect)
    int y = SvIV(ST(1));
    int wd = SvIV(ST(2));
    int ht = SvIV(ST(3));
-   // check that wd & ht are > 0
-   if (wd <= 0) PERL_ERROR("g_visrect error: width must be > 0.");
-   if (ht <= 0) PERL_ERROR("g_visrect error: height must be > 0.");
+   const char* err = GSF_checkrect(x, y, wd, ht);
+   if (err) PERL_ERROR(err);
 
    bigint left = x;
    bigint top = y;
@@ -3017,6 +3118,8 @@ EXTERN_C void xs_init(pTHX)
    newXS((char*)"g_getalgo",      pl_getalgo,      (char*)file);
    newXS((char*)"g_setrule",      pl_setrule,      (char*)file);
    newXS((char*)"g_getrule",      pl_getrule,      (char*)file);
+   newXS((char*)"g_getwidth",     pl_getwidth,     (char*)file);
+   newXS((char*)"g_getheight",    pl_getheight,    (char*)file);
    // viewing
    newXS((char*)"g_setpos",       pl_setpos,       (char*)file);
    newXS((char*)"g_getpos",       pl_getpos,       (char*)file);
@@ -3080,12 +3183,16 @@ void RunPerlScript(const wxString &filepath)
             if ( !LoadPerlLib() ) return;
          }
       #endif
-   
+
+      // create a dummy environment for initializing the embedded interpreter
+      static int argc = 3;
+      static char arg1[] = "-e", arg2[] = "0";
+      static char *args[] = { NULL, arg1, arg2, NULL }, **argv = &args[0];
+
       #ifdef PERL510_OR_LATER
+         static char *ens[] = { NULL }, **env = &ens[0];
          if (!inited) {
-            int argc = 3;
-            static const char* argv[] = { "", "-e", "" };
-            Perl_sys_init3(&argc, (char***)&argv, NULL);
+            PERL_SYS_INIT3(&argc, &argv, &env);
             inited = true;
          }
       #endif
@@ -3096,14 +3203,14 @@ void RunPerlScript(const wxString &filepath)
          return;
       }
    
+      PL_perl_destruct_level = 1;
       perl_construct(my_perl);
    
       // set PERL_EXIT_DESTRUCT_END flag so that perl_destruct will execute
       // any END blocks in given script (this flag requires Perl 5.7.2+)
       PL_exit_flags |= PERL_EXIT_DESTRUCT_END;
    
-      static const char* embedding[] = { "", "-e", "" };
-      perl_parse(my_perl, xs_init, 3, (char**)embedding, NULL);
+      perl_parse(my_perl, xs_init, argc, argv, NULL);
       perl_run(my_perl);
    }
 
@@ -3123,6 +3230,7 @@ void RunPerlScript(const wxString &filepath)
       // or if g_exit has been called
       wxString savestring = scripterr;
       scripterr = wxEmptyString;
+      PL_perl_destruct_level = 1;
       perl_destruct(my_perl);
       scripterr = savestring;
 
@@ -3136,7 +3244,9 @@ void RunPerlScript(const wxString &filepath)
 void FinishPerlScripting()
 {
    #ifdef PERL510_OR_LATER
-      if (inited) Perl_sys_term();
+      if (inited) {
+         PERL_SYS_TERM();
+      }
    #endif
 
    #ifdef USE_PERL_DYNAMIC
