@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -86,6 +86,7 @@ extern bool showedit;            // show edit bar?
 extern bool showallstates;       // show all cell states in edit bar?
 extern bool showstatus;          // show status bar?
 extern bool showexact;           // show exact numbers in status bar?
+extern bool showtimeline;        // show timeline bar?
 extern bool showgridlines;       // display grid lines?
 extern bool showicons;           // display icons for cell states?
 extern bool swapcolors;          // swap colors used for cell states?
@@ -95,6 +96,7 @@ extern bool scrollcross;         // scroll if cross cursor is dragged outside vi
 extern bool scrollhand;          // scroll if hand cursor is dragged outside view?
 extern bool allowundo;           // allow undo/redo?
 extern bool restoreview;         // should reset/undo restore view?
+extern bool allowbeep;           // okay to play beep sound?
 extern int controlspos;          // position of translucent controls
 extern int canchangerule;        // if > 0 then paste can change rule
 extern int randomfill;           // random fill percentage
@@ -174,6 +176,7 @@ typedef enum {
    DO_PASTEMODE,                 // cycle paste mode
    DO_DELETE,                    // delete layer
    DO_DELOTHERS,                 // delete other layers
+   DO_DELTIME,                   // delete timeline
    DO_DISABLE,                   // disable undo/redo
    DO_DUPLICATE,                 // duplicate layer
    DO_FASTER,                    // faster
@@ -190,11 +193,14 @@ typedef enum {
    DO_NEWPATT,                   // new pattern
    DO_NEXTGEN,                   // next generation
    DO_NEXTSTEP,                  // next step
+   DO_NEXTHIGHER,                // next higher state
+   DO_NEXTLOWER,                 // next lower state
    DO_OPENCLIP,                  // open clipboard
    DO_OPENPATT,                  // open pattern...
    DO_PASTE,                     // paste
    DO_PASTESEL,                  // paste to selection
    DO_INFO,                      // pattern info
+   DO_PLAYBACK,                  // play timeline backwards
    DO_PREFS,                     // preferences...
    DO_QUIT,                      // quit Golly
    DO_RANDFILL,                  // random fill
@@ -239,6 +245,7 @@ typedef enum {
    DO_PATTERNS,                  // show patterns
    DO_SCRIPTS,                   // show scripts
    DO_SHOWSTATUS,                // show status bar
+   DO_SHOWTIME,                  // show timeline
    DO_TIMING,                    // show timing
    DO_SHOWTOOL,                  // show tool bar
    DO_SHRINKFIT,                 // shrink and fit
@@ -246,6 +253,7 @@ typedef enum {
    DO_SLOWER,                    // slower
    DO_STACK,                     // stack layers
    DO_STARTSTOP,                 // start/stop generating
+   DO_RECORD,                    // start/stop recording
    DO_SYNCCURS,                  // synchronize cursors
    DO_SYNCVIEWS,                 // synchronize views
    DO_TILE,                      // tile layers
@@ -279,8 +287,10 @@ wxString GetShortcutTable();
 
 // Colors:
 
+extern wxColor* borderrgb;       // color for border around bounded grid
 extern wxColor* selectrgb;       // color for selected cells
 extern wxColor* pastergb;        // color for pasted pattern
+extern wxBrush* borderbrush;     // brush for filling grid border
 extern wxPen* pastepen;          // pen for drawing paste rect
 
 void SetBrushesAndPens();        // update colors in above brushes and pens
@@ -304,7 +314,7 @@ const int MAX_SPACING = 1000;    // maximum value of boldspacing
 const int MIN_MEM_MB = 0;        // minimum value of maximum memory
 const int MAX_MEM_MB =           // maximum value of maximum memory
             sizeof(char*) <= 4 ? 4000 : 100000000;
-const int MAX_BASESTEP = 10000;  // maximum base step
+const int MAX_BASESTEP = 2000000000;  // maximum base step
 const int MAX_DELAY = 5000;      // maximum mindelay or maxdelay
 const int MAX_THUMBRANGE = 500;  // maximum thumbrange
 const int MIN_DIRWD = 50;        // minimum dirwinwd
