@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -106,7 +106,11 @@ extern "C"
 {
    // startup/shutdown
    void(*G_Py_Initialize)(void) = NULL;
+#if defined(__LP64__) || defined(__amd64__)
+   PyObject*(*G_Py_InitModule4_64)(char*, struct PyMethodDef*, char*, PyObject*, int) = NULL;
+#else
    PyObject*(*G_Py_InitModule4)(char*, struct PyMethodDef*, char*, PyObject*, int) = NULL;
+#endif
    void(*G_Py_Finalize)(void) = NULL;
 
    // errors
@@ -145,7 +149,11 @@ extern "C"
 
 // redefine the Py* functions to their equivalent G_* wrappers
 #define Py_Initialize         G_Py_Initialize
-#define Py_InitModule4        G_Py_InitModule4
+#if defined(__LP64__) || defined(__amd64__)
+   #define Py_InitModule4_64  G_Py_InitModule4_64
+#else
+   #define Py_InitModule4     G_Py_InitModule4
+#endif
 #define Py_Finalize           G_Py_Finalize
 #define PyErr_Occurred        G_PyErr_Occurred
 #define PyErr_SetString       G_PyErr_SetString
@@ -185,7 +193,11 @@ static struct PythonFunc
 } pythonFuncs[] =
 {
    PYTHON_FUNC(Py_Initialize)
+#if defined(__LP64__) || defined(__amd64__)
+   PYTHON_FUNC(Py_InitModule4_64)
+#else
    PYTHON_FUNC(Py_InitModule4)
+#endif
    PYTHON_FUNC(Py_Finalize)
    PYTHON_FUNC(PyErr_Occurred)
    PYTHON_FUNC(PyErr_SetString)
@@ -254,14 +266,15 @@ static bool LoadPythonLib()
    // is needed to avoid an ImportError when importing some modules (eg. time)
    while ( !dynlib.Load(pythonlib, wxDL_NOW | wxDL_VERBATIM | wxDL_GLOBAL) ) {
       // prompt user for a different Python library;
-      // on Windows pythonlib should be something like "python25.dll"
-      // and on Linux it should be something like "libpython2.5.so"
-      wxBell();
+      // on Windows pythonlib should be something like "python27.dll"
+      // and on Linux it should be something like "libpython2.7.so"
+      Beep();
       wxString str = _("If Python isn't installed then you'll have to Cancel,");
-      str +=         _("\notherwise change the version numbers and try again.");
+      str +=         _("\notherwise change the version numbers to match the");
+      str +=         _("\nversion installed on your system and try again.");
       #ifdef __WXMSW__
-         str +=      _("\nDepending on where you installed Python you might have");
-         str +=      _("\nto enter a full path like C:\\Python25\\python25.dll.");
+         str +=      _("\n\nIf that fails, search your system for a python*.dll");
+         str +=      _("\nfile and enter the full path to that file.");
       #endif
       wxTextEntryDialog dialog( wxGetActiveWindow(), str,
                                 _("Could not load the Python library"),
@@ -645,6 +658,9 @@ static PyObject* py_store(PyObject* self, PyObject* args)
       int item = ints_per_cell * n;
       long x = PyInt_AsLong( PyList_GetItem(inlist, item) );
       long y = PyInt_AsLong( PyList_GetItem(inlist, item + 1) );
+      // check if x,y is outside bounded grid
+      const char* err = GSF_checkpos(tempalgo, x, y);
+      if (err) { delete tempalgo; PYTHON_ERROR(err); }
       if (multistate) {
          long state = PyInt_AsLong( PyList_GetItem(inlist, item + 2) );
          if (tempalgo->setcell(x, y, state) < 0) {
@@ -666,8 +682,10 @@ static PyObject* py_store(PyObject* self, PyObject* args)
    // write pattern to given file in RLE/XRLE format
    bigint top, left, bottom, right;
    tempalgo->findedges(&top, &left, &bottom, &right);
-   const char* err = writepattern(FILENAME, *tempalgo,
-                        savexrle ? XRLE_format : RLE_format,
+   pattern_format format = savexrle ? XRLE_format : RLE_format;
+   // if grid is bounded then force XRLE_format so that position info is recorded
+   if (tempalgo->gridwd > 0 || tempalgo->gridht > 0) format = XRLE_format;
+   const char* err = writepattern(FILENAME, *tempalgo, format,
                         top.toint(), left.toint(), bottom.toint(), right.toint());
    delete tempalgo;
    if (err) PYTHON_ERROR(err);
@@ -1083,6 +1101,9 @@ static PyObject* py_evolve(PyObject* self, PyObject* args)
       int item = ints_per_cell * n;
       long x = PyInt_AsLong( PyList_GetItem(inlist, item) );
       long y = PyInt_AsLong( PyList_GetItem(inlist, item + 1) );
+      // check if x,y is outside bounded grid
+      const char* err = GSF_checkpos(tempalgo, x, y);
+      if (err) { delete tempalgo; PYTHON_ERROR(err); }
       if (multistate) {
          long state = PyInt_AsLong( PyList_GetItem(inlist, item + 2) );
          if (tempalgo->setcell(x, y, state) < 0) {
@@ -1103,8 +1124,25 @@ static PyObject* py_evolve(PyObject* self, PyObject* args)
 
    // advance pattern by ngens
    mainptr->generating = true;
-   tempalgo->setIncrement(ngens);
-   tempalgo->step();
+   if (tempalgo->gridwd > 0 || tempalgo->gridht > 0) {
+      // a bounded grid must use an increment of 1 so we can call
+      // CreateBorderCells and DeleteBorderCells around each step()
+      tempalgo->setIncrement(1);
+      while (ngens > 0) {
+         if (PythonScriptAborted()) {
+            mainptr->generating = false;
+            delete tempalgo;
+            return NULL;
+         }
+         if (!mainptr->CreateBorderCells(tempalgo)) break;
+         tempalgo->step();
+         if (!mainptr->DeleteBorderCells(tempalgo)) break;
+         ngens--;
+      }
+   } else {
+      tempalgo->setIncrement(ngens);
+      tempalgo->step();
+   }
    mainptr->generating = false;
 
    // convert new pattern into a new cell list
@@ -1164,6 +1202,7 @@ static PyObject* py_putcells(PyObject* self, PyObject* args)
    int ints_per_cell = multistate ? 3 : 2;
    int num_cells = PyList_Size(list) / ints_per_cell;
    bool abort = false;
+   const char* err = NULL;
    bool pattchanged = false;
    lifealgo* curralgo = currlayer->algo;
 
@@ -1180,6 +1219,9 @@ static PyObject* py_putcells(PyObject* self, PyObject* args)
             long y = PyInt_AsLong( PyList_GetItem(list, item + 1) );
             int newx = x0 + x * axx + y * axy;
             int newy = y0 + x * ayx + y * ayy;
+            // check if newx,newy is outside bounded grid
+            err = GSF_checkpos(curralgo, newx, newy);
+            if (err) break;
             int oldstate = curralgo->getcell(newx, newy);
             if (multistate) {
                // multi-state lists can contain dead cells so newstate might be 0
@@ -1205,6 +1247,9 @@ static PyObject* py_putcells(PyObject* self, PyObject* args)
          long y = PyInt_AsLong( PyList_GetItem(list, item + 1) );
          int newx = x0 + x * axx + y * axy;
          int newy = y0 + x * ayx + y * ayy;
+         // check if newx,newy is outside bounded grid
+         err = GSF_checkpos(curralgo, newx, newy);
+         if (err) break;
          int oldstate = curralgo->getcell(newx, newy);
          int newstate;
          if (multistate) {
@@ -1255,6 +1300,9 @@ static PyObject* py_putcells(PyObject* self, PyObject* args)
          long y = PyInt_AsLong( PyList_GetItem(list, item + 1) );
          int newx = x0 + x * axx + y * axy;
          int newy = y0 + x * ayx + y * ayy;
+         // check if newx,newy is outside bounded grid
+         err = GSF_checkpos(curralgo, newx, newy);
+         if (err) break;
          int oldstate = curralgo->getcell(newx, newy);
          if (multistate) {
             // multi-state lists can contain dead cells so newstate might be 0
@@ -1285,6 +1333,7 @@ static PyObject* py_putcells(PyObject* self, PyObject* args)
       DoAutoUpdate();
    }
 
+   if (err) PYTHON_ERROR(err);
    if (abort) return NULL;
 
    RETURN_NONE;
@@ -1311,14 +1360,10 @@ static PyObject* py_getcells(PyObject* self, PyObject* args)
       int itop = PyInt_AsLong( PyList_GetItem(rect_list, 1) );
       int wd = PyInt_AsLong( PyList_GetItem(rect_list, 2) );
       int ht = PyInt_AsLong( PyList_GetItem(rect_list, 3) );
-      // first check that wd & ht are > 0
-      if (wd <= 0) {
+      const char* err = GSF_checkrect(ileft, itop, wd, ht);
+      if (err) {
          Py_DECREF(outlist);
-         PYTHON_ERROR("getcells error: width must be > 0.");
-      }
-      if (ht <= 0) {
-         Py_DECREF(outlist);
-         PYTHON_ERROR("getcells error: height must be > 0.");
+         PYTHON_ERROR(err);
       }
       int iright = ileft + wd - 1;
       int ibottom = itop + ht - 1;
@@ -1439,38 +1484,10 @@ static PyObject* py_hash(PyObject* self, PyObject* args)
    int y  = PyInt_AsLong( PyList_GetItem(rect_list, 1) );
    int wd = PyInt_AsLong( PyList_GetItem(rect_list, 2) );
    int ht = PyInt_AsLong( PyList_GetItem(rect_list, 3) );
-   // first check that wd & ht are > 0
-   if (wd <= 0) PYTHON_ERROR("hash error: width must be > 0.");
-   if (ht <= 0) PYTHON_ERROR("hash error: height must be > 0.");
-   int right = x + wd - 1;
-   int bottom = y + ht - 1;
-   int cx, cy;
-   int v = 0;
-   int cntr = 0;
+   const char* err = GSF_checkrect(x, y, wd, ht);
+   if (err) PYTHON_ERROR(err);
 
-   // calculate a hash value for pattern in given rect
-   int hash = 31415962;
-   lifealgo* curralgo = currlayer->algo;
-   for ( cy=y; cy<=bottom; cy++ ) {
-      int yshift = cy - y;
-      for ( cx=x; cx<=right; cx++ ) {
-         int skip = curralgo->nextcell(cx, cy, v);
-         if (skip >= 0) {
-            // found next live cell in this row
-            cx += skip;
-            if (cx <= right) {
-               //note that v is 1 in a two-state universe
-               hash = (hash * 33 + yshift) ^ ((cx - x) * v);
-            }
-         } else {
-            cx = right;  // done this row
-         }
-         cntr++;
-         if ((cntr % 4096) == 0 && PythonScriptAborted()) {
-            return NULL;
-         }
-      }
-   }
+   int hash = GSF_hash(x, y, wd, ht);
 
    return Py_BuildValue((char*)"i", hash);
 }
@@ -1575,9 +1592,8 @@ static PyObject* py_select(PyObject* self, PyObject* args)
       int y  = PyInt_AsLong( PyList_GetItem(rect_list, 1) );
       int wd = PyInt_AsLong( PyList_GetItem(rect_list, 2) );
       int ht = PyInt_AsLong( PyList_GetItem(rect_list, 3) );
-      // first check that wd & ht are > 0
-      if (wd <= 0) PYTHON_ERROR("select error: width must be > 0.");
-      if (ht <= 0) PYTHON_ERROR("select error: height must be > 0.");
+      const char* err = GSF_checkrect(x, y, wd, ht);
+      if (err) PYTHON_ERROR(err);
       // set selection rect
       GSF_select(x, y, wd, ht);
    } else {
@@ -1671,6 +1687,10 @@ static PyObject* py_getcell(PyObject* self, PyObject* args)
 
    if (!PyArg_ParseTuple(args, (char*)"ii", &x, &y)) return NULL;
 
+   // check if x,y is outside bounded grid
+   const char* err = GSF_checkpos(currlayer->algo, x, y);
+   if (err) PYTHON_ERROR(err);
+
    return Py_BuildValue((char*)"i", currlayer->algo->getcell(x, y));
 }
 
@@ -1975,6 +1995,30 @@ static PyObject* py_getrule(PyObject* self, PyObject* args)
 
 // -----------------------------------------------------------------------------
 
+static PyObject* py_getwidth(PyObject* self, PyObject* args)
+{
+   if (PythonScriptAborted()) return NULL;
+   wxUnusedVar(self);
+
+   if (!PyArg_ParseTuple(args, (char*)"")) return NULL;
+
+   return Py_BuildValue((char*)"i", currlayer->algo->gridwd);
+}
+
+// -----------------------------------------------------------------------------
+
+static PyObject* py_getheight(PyObject* self, PyObject* args)
+{
+   if (PythonScriptAborted()) return NULL;
+   wxUnusedVar(self);
+
+   if (!PyArg_ParseTuple(args, (char*)"")) return NULL;
+
+   return Py_BuildValue((char*)"i", currlayer->algo->gridht);
+}
+
+// -----------------------------------------------------------------------------
+
 static PyObject* py_numstates(PyObject* self, PyObject* args)
 {
    if (PythonScriptAborted()) return NULL;
@@ -2115,9 +2159,8 @@ static PyObject* py_visrect(PyObject* self, PyObject* args)
    int y = PyInt_AsLong( PyList_GetItem(rect_list, 1) );
    int wd = PyInt_AsLong( PyList_GetItem(rect_list, 2) );
    int ht = PyInt_AsLong( PyList_GetItem(rect_list, 3) );
-   // check that wd & ht are > 0
-   if (wd <= 0) PYTHON_ERROR("visrect error: width must be > 0.");
-   if (ht <= 0) PYTHON_ERROR("visrect error: height must be > 0.");
+   const char* err = GSF_checkrect(x, y, wd, ht);
+   if (err) PYTHON_ERROR(err);
 
    bigint left = x;
    bigint top = y;
@@ -2828,6 +2871,8 @@ static PyMethodDef py_methods[] = {
    { "getalgo",      py_getalgo,    METH_VARARGS, "return name of given or current algorithm" },
    { "setrule",      py_setrule,    METH_VARARGS, "set current rule using given string" },
    { "getrule",      py_getrule,    METH_VARARGS, "return current rule" },
+   { "getwidth",     py_getwidth,   METH_VARARGS, "return width of universe (0 if unbounded)" },
+   { "getheight",    py_getheight,  METH_VARARGS, "return height of universe (0 if unbounded)" },
    // viewing
    { "setpos",       py_setpos,     METH_VARARGS, "move given cell to middle of viewport" },
    { "getpos",       py_getpos,     METH_VARARGS, "return x,y position of cell in middle of viewport" },
