@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -59,6 +59,8 @@ DrawView() does the following tasks:
 
 - Calls DrawGridLines() to overlay grid lines if they are visible.
 
+- Calls DrawGridBorder() to draw border around a bounded universe.
+
 - Calls DrawSelection() to overlay a translucent selection rectangle
   if a selection exists and any part of it is visible.
 
@@ -93,7 +95,7 @@ Other points of interest:
 - The decision to use buffered drawing is made in PatternView::OnPaint().
   It's never used on Mac OS X or GTK+ 2.0 because windows on those systems
   are automatically buffered.  To avoid flicker on Windows, buffering is
-  always used if:
+  always used if any of these conditions are true:
   - the user is doing a paste;
   - the grid lines are visible;
   - the selection is visible;
@@ -123,7 +125,7 @@ Other points of interest:
 
 #include "wxgolly.h"       // for viewptr, bigview, statusptr
 #include "wxutils.h"       // for Warning, Fatal, FillRect
-#include "wxprefs.h"       // for swapcolors, showgridlines, mingridmag, etc
+#include "wxprefs.h"       // for showgridlines, mingridmag, swapcolors, etc
 #include "wxstatus.h"      // for statusptr->...
 #include "wxview.h"        // for viewptr->...
 #include "wxlayer.h"       // currlayer, GetLayer, etc
@@ -155,6 +157,7 @@ int pastemag;                    // must match current viewport's scale
 int cvwd, cvht;                  // must match current viewport's width and height
 paste_location pasteloc;         // must match plocation
 bool pasteicons;                 // must match showicons
+bool pastecolors;                // must match swapcolors
 lifealgo* pastealgo;             // universe containing paste pattern
 wxRect pastebbox;                // bounding box in cell coords (not necessarily minimal)
 
@@ -558,7 +561,8 @@ void DrawIcons(unsigned char* byteptr, int x, int y, int w, int h, int pmscale)
                wxAlphaPixelData::Iterator topleft = p;
                if (state && iconmaps[state]) {
                   // copy cellsize*cellsize pixels from iconmaps[state]
-                  // but convert black pixels to dead cell color;
+                  // but convert black pixels to dead cell color
+                  bool multicolor = iconmaps[1]->GetDepth() > 1;
                   #ifdef __WXMSW__
                      // must use wxNativePixelData for bitmaps with no alpha channel
                      wxNativePixelData icondata(*iconmaps[state]);
@@ -580,10 +584,23 @@ void DrawIcons(unsigned char* byteptr, int x, int y, int w, int h, int pmscale)
                         #endif
                         for (int j = 0; j < cellsize; j++) {
                            if (iconpxl.Red() || iconpxl.Green() || iconpxl.Blue()) {
-                              // replace non-black pixel with current cell color
-                              p.Red()   = currlayer->cellr[state];
-                              p.Green() = currlayer->cellg[state];
-                              p.Blue()  = currlayer->cellb[state];
+                              if (multicolor) {
+                                 // use non-black pixel in multi-colored icon
+                                 if (swapcolors) {
+                                    p.Red()   = 255 - iconpxl.Red();
+                                    p.Green() = 255 - iconpxl.Green();
+                                    p.Blue()  = 255 - iconpxl.Blue();
+                                 } else {
+                                    p.Red()   = iconpxl.Red();
+                                    p.Green() = iconpxl.Green();
+                                    p.Blue()  = iconpxl.Blue();
+                                 }
+                              } else {
+                                 // replace non-black pixel with current cell color
+                                 p.Red()   = currlayer->cellr[state];
+                                 p.Green() = currlayer->cellg[state];
+                                 p.Blue()  = currlayer->cellb[state];
+                              }
                            } else {
                               // replace black pixel with dead cell color
                               p.Red()   = deadred;
@@ -686,6 +703,7 @@ void DrawOneIcon(wxDC& dc, int x, int y, wxBitmap* icon,
    // and convert non-black pixels to given live cell color
    int wd = icon->GetWidth();
    int ht = icon->GetHeight();
+   bool multicolor = icon->GetDepth() > 1;
    wxBitmap pixmap(wd, ht, 32);
 
    wxAlphaPixelData pxldata(pixmap);
@@ -718,10 +736,23 @@ void DrawOneIcon(wxDC& dc, int x, int y, wxBitmap* icon,
             #endif
             for (int j = 0; j < wd; j++) {
                if (iconpxl.Red() || iconpxl.Green() || iconpxl.Blue()) {
-                  // replace non-black pixel with live cell color
-                  p.Red()   = liver;
-                  p.Green() = liveg;
-                  p.Blue()  = liveb;
+                  if (multicolor) {
+                     // use non-black pixel in multi-colored icon
+                     if (swapcolors) {
+                        p.Red()   = 255 - iconpxl.Red();
+                        p.Green() = 255 - iconpxl.Green();
+                        p.Blue()  = 255 - iconpxl.Blue();
+                     } else {
+                        p.Red()   = iconpxl.Red();
+                        p.Green() = iconpxl.Green();
+                        p.Blue()  = iconpxl.Blue();
+                     }
+                  } else {
+                     // replace non-black pixel with live cell color
+                     p.Red()   = liver;
+                     p.Green() = liveg;
+                     p.Blue()  = liveb;
+                  }
                } else {
                   // replace black pixel with dead cell color
                   p.Red()   = deadr;
@@ -943,6 +974,7 @@ void CreatePasteImage(lifealgo* palgo, wxRect& bbox)
    pimageht = -1;
    pastemag = currlayer->view->getmag();
    pasteicons = showicons;
+   pastecolors = swapcolors;
 }
 
 // -----------------------------------------------------------------------------
@@ -1025,14 +1057,15 @@ int PixelsToCells(int pixels) {
 
 void CheckPasteImage()
 {
-   // paste image needs to be updated if pasterect size changed
-   // or viewport size changed or plocation changed or showicons changed
+   // paste image needs to be updated if any of these changed:
+   // pasterect size, viewport size, plocation, showicons, swapcolors
    if ( prectwd != viewptr->pasterect.width ||
         prectht != viewptr->pasterect.height ||
         cvwd != currlayer->view->getwidth() ||
         cvht != currlayer->view->getheight() ||
         pasteloc != plocation ||
-        pasteicons != showicons
+        pasteicons != showicons ||
+        pastecolors != swapcolors
       ) {
       prectwd = viewptr->pasterect.width;
       prectht = viewptr->pasterect.height;
@@ -1041,6 +1074,7 @@ void CheckPasteImage()
       cvht = currlayer->view->getheight();
       pasteloc = plocation;
       pasteicons = showicons;
+      pastecolors = swapcolors;
 
       // calculate size of paste image; we could just set it to pasterect size
       // but that would be slow and wasteful for large pasterects, so we use
@@ -1256,7 +1290,7 @@ void DrawPasteImage(wxDC& dc)
 
 // -----------------------------------------------------------------------------
 
-void DrawGridLines(wxDC& dc, wxRect& r)
+void DrawGridLines(wxDC& dc)
 {
    int cellsize = 1 << currlayer->view->getmag();
    int h, v, i, topbold, leftbold;
@@ -1275,7 +1309,7 @@ void DrawGridLines(wxDC& dc, wxRect& r)
       }
       if (mathcoords) topbold--;   // show origin cell above bold line
    } else {
-      // avoid spurious gcc warning
+      // avoid gcc warning
       topbold = leftbold = 0;
    }
 
@@ -1286,19 +1320,19 @@ void DrawGridLines(wxDC& dc, wxRect& r)
    v = -1;
    while (true) {
       v += cellsize;
-      if (v >= r.height) break;
+      if (v >= currht) break;
       if (showboldlines) i++;
-      if (i % boldspacing != 0 && v >= r.y && v < r.y + r.height)
-         dc.DrawLine(r.x, v, r.GetRight() + 1, v);
+      if (i % boldspacing != 0 && v >= 0 && v < currht)
+         dc.DrawLine(0, v, currwd, v);
    }
    i = showboldlines ? leftbold : 1;
    h = -1;
    while (true) {
       h += cellsize;
-      if (h >= r.width) break;
+      if (h >= currwd) break;
       if (showboldlines) i++;
-      if (i % boldspacing != 0 && h >= r.x && h < r.x + r.width)
-         dc.DrawLine(h, r.y, h, r.GetBottom() + 1);
+      if (i % boldspacing != 0 && h >= 0 && h < currwd)
+         dc.DrawLine(h, 0, h, currht);
    }
 
    if (showboldlines) {
@@ -1308,19 +1342,19 @@ void DrawGridLines(wxDC& dc, wxRect& r)
       v = -1;
       while (true) {
          v += cellsize;
-         if (v >= r.height) break;
+         if (v >= currht) break;
          i++;
-         if (i % boldspacing == 0 && v >= r.y && v < r.y + r.height)
-            dc.DrawLine(r.x, v, r.GetRight() + 1, v);
+         if (i % boldspacing == 0 && v >= 0 && v < currht)
+            dc.DrawLine(0, v, currwd, v);
       }
       i = leftbold;
       h = -1;
       while (true) {
          h += cellsize;
-         if (h >= r.width) break;
+         if (h >= currwd) break;
          i++;
-         if (i % boldspacing == 0 && h >= r.x && h < r.x + r.width)
-            dc.DrawLine(h, r.y, h, r.GetBottom() + 1);
+         if (i % boldspacing == 0 && h >= 0 && h < currwd)
+            dc.DrawLine(h, 0, h, currht);
       }
    }
    
@@ -1329,10 +1363,106 @@ void DrawGridLines(wxDC& dc, wxRect& r)
 
 // -----------------------------------------------------------------------------
 
+void DrawGridBorder(wxDC& dc)
+{
+   // universe is bounded so draw any visible border regions
+   pair<int,int> ltpxl = currlayer->view->screenPosOf(currlayer->algo->gridleft,
+                                                      currlayer->algo->gridtop,
+                                                      currlayer->algo);
+   pair<int,int> rbpxl = currlayer->view->screenPosOf(currlayer->algo->gridright,
+                                                      currlayer->algo->gridbottom,
+                                                      currlayer->algo);
+   int left = ltpxl.first;
+   int top = ltpxl.second;
+   int right = rbpxl.first;
+   int bottom = rbpxl.second;
+   if (currlayer->algo->gridwd == 0) {
+      left = 0;
+      right = currwd-1;
+   }
+   if (currlayer->algo->gridht == 0) {
+      top = 0;
+      bottom = currht-1;
+   }
+
+   // note that right and/or bottom might be INT_MAX so avoid adding to cause overflow
+   if (currlayer->view->getmag() > 0) {
+      // move to bottom right pixel of cell at gridright,gridbottom
+      if (right < currwd) right += (1 << currlayer->view->getmag()) - 1;
+      if (bottom < currht) bottom += (1 << currlayer->view->getmag()) - 1;
+      if (currlayer->view->getmag() == 1) {
+         // there are no gaps at scale 1:2
+         if (right < currwd) right++;
+         if (bottom < currht) bottom++;
+      }
+   } else {
+      if (right < currwd) right++;
+      if (bottom < currht) bottom++;
+   }
+
+   if (left < 0 && right >= currwd && top < 0 && bottom >= currht) {
+      // border isn't visible (ie. grid fills viewport)
+      return;
+   }
+
+   if (left >= currwd || right < 0 || top >= currht || bottom < 0) {
+      // no part of grid is visible so fill viewport with border
+      wxRect r(0, 0, currwd, currht);
+      FillRect(dc, r, *borderbrush);
+      return;
+   }
+   
+   // avoid drawing overlapping rects below
+   int rtop = 0;
+   int rheight = currht;
+   
+   if (currlayer->algo->gridht > 0) {
+      if (top > 0) {
+         // top border is visible
+         wxRect r(0, 0, currwd, top);
+         FillRect(dc, r, *borderbrush);
+         // reduce size of rect below
+         rtop = top;
+         rheight -= top;
+      }
+      if (bottom < currht) {
+         // bottom border is visible
+         wxRect r(0, bottom, currwd, currht - bottom);
+         FillRect(dc, r, *borderbrush);
+         // reduce size of rect below
+         rheight -= currht - bottom;
+      }
+   }
+
+   if (currlayer->algo->gridwd > 0) {
+      if (left > 0) {
+         // left border is visible
+         wxRect r(0, rtop, left, rheight);
+         FillRect(dc, r, *borderbrush);
+      }
+      if (right < currwd) {
+         // right border is visible
+         wxRect r(right, rtop, currwd - right, rheight);
+         FillRect(dc, r, *borderbrush);
+      }
+   }
+}
+
+// -----------------------------------------------------------------------------
+
 void DrawOneLayer(wxDC& dc)
 {
    wxMemoryDC layerdc;
    layerdc.SelectObject(*layerbitmap);
+
+   // only show icons at scales 1:8 and 1:16
+   if (showicons && currlayer->view->getmag() > 2) {
+      if (currlayer->view->getmag() == 3) {
+         iconmaps = currlayer->icons7x7;
+      } else {
+         iconmaps = currlayer->icons15x15;
+      }
+   }
    
    currdc = &layerdc;
    currlayer->algo->draw(*currlayer->view, renderer);
@@ -1468,12 +1598,39 @@ void DrawView(wxDC& dc, int tileindex)
    viewport* saveview0 = NULL;
    int colorindex;
    
+   // if grid is bounded then ensure viewport's central cell is not outside grid edges
+   if ( currlayer->algo->gridwd > 0) {
+      if ( currlayer->view->x < currlayer->algo->gridleft )
+         currlayer->view->setpositionmag(currlayer->algo->gridleft,
+                                         currlayer->view->y,
+                                         currlayer->view->getmag());
+      else if ( currlayer->view->x > currlayer->algo->gridright )
+         currlayer->view->setpositionmag(currlayer->algo->gridright,
+                                         currlayer->view->y,
+                                         currlayer->view->getmag());
+   }
+   if ( currlayer->algo->gridht > 0) {
+      if ( currlayer->view->y < currlayer->algo->gridtop )
+         currlayer->view->setpositionmag(currlayer->view->x,
+                                         currlayer->algo->gridtop,
+                                         currlayer->view->getmag());
+      else if ( currlayer->view->y > currlayer->algo->gridbottom )
+         currlayer->view->setpositionmag(currlayer->view->x,
+                                         currlayer->algo->gridbottom,
+                                         currlayer->view->getmag());
+   }
+   
    if ( viewptr->nopattupdate ) {
       // don't draw incomplete pattern, just fill background
-      r = wxRect(0, 0, currlayer->view->getwidth(), currlayer->view->getheight());
+      currwd = currlayer->view->getwidth();
+      currht = currlayer->view->getheight();
+      r = wxRect(0, 0, currwd, currht);
       FillRect(dc, r, *currlayer->deadbrush);
-      // might as well draw grid lines
-      if ( viewptr->GridVisible() ) DrawGridLines(dc, r);
+      // might as well draw grid lines and border
+      if ( viewptr->GridVisible() )
+         DrawGridLines(dc);         // uses currwd and currht
+      if ( currlayer->algo->gridwd > 0 || currlayer->algo->gridht > 0 )
+         DrawGridBorder(dc);        // uses currwd and currht
       return;
    }
 
@@ -1524,12 +1681,16 @@ void DrawView(wxDC& dc, int tileindex)
    currlayer->algo->draw(*currlayer->view, renderer);
 
    if ( viewptr->GridVisible() ) {
-      r = wxRect(0, 0, currlayer->view->getwidth(), currlayer->view->getheight());
-      DrawGridLines(dc, r);
+      DrawGridLines(dc);      // uses currwd and currht
+   }
+
+   // if universe is bounded then draw border regions (if visible)
+   if ( currlayer->algo->gridwd > 0 || currlayer->algo->gridht > 0 ) {
+      DrawGridBorder(dc);     // uses currwd and currht
    }
    
    if ( currlayer->currsel.Visible(&r) ) {
-      CheckSelectionSize(currlayer->view->getwidth(), currlayer->view->getheight());
+      CheckSelectionSize(currwd, currht);
       if (colorindex == currindex)
          DrawSelection(dc, r);
       else
