@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -34,7 +34,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include "wxgolly.h"       // for wxGetApp, mainptr
 #include "wxmain.h"        // for mainptr->...
 #include "wxprefs.h"       // for namedrules, gollydir, showalgohelp, etc
-#include "wxutils.h"       // for Warning, Fatal
+#include "wxutils.h"       // for Warning, Fatal, Beep
 #include "wxlayer.h"       // for currlayer
 #include "wxalgos.h"       // for NumAlgos, CreateNewUniverse, etc
 #include "wxrule.h"
@@ -72,43 +72,46 @@ bool ValidRule(wxString& rule)
 
 // -----------------------------------------------------------------------------
 
-bool MatchingRules(const wxString& rule1, const wxString& rule2)
-{
-   // return true if given strings are equivalent rules
-   if (rule1.IsSameAs(rule2,false)) {
-      return true;
-   } else {
-      /* avoid calling ValidRule -- all the setrule calls cause problems
-         for algos like RuleTable and RuleTree that load files;
-         or maybe we could pass an extra arg into setrule saying not to try
-         and load a file but just check for its existence???
-         
-      // we want "s23b3" or "23/3" to match "B3/S23" so convert given rules
-      // to canonical form (if valid) and then compare
-      wxString canon1 = rule1;
-      wxString canon2 = rule2;
-      return ValidRule(canon1) && ValidRule(canon2) && canon1 == canon2;
-      */
-      return false;
-   }
-}
-
-// -----------------------------------------------------------------------------
-
 wxString GetRuleName(const wxString& rulestring)
 {
    // search namedrules array for matching rule
    wxString rulename;
-   size_t i;
-   for (i = 0; i < namedrules.GetCount(); i++) {
-      // extract rule after '|'
-      wxString thisrule = namedrules[i].AfterFirst('|');
-      if ( MatchingRules(rulestring, thisrule) ) {
-         // extract name before '|'
-         rulename = namedrules[i].BeforeFirst('|');
+   
+   // check for a suffix like ":T100,200"
+   wxString ruleprefix = rulestring;
+   wxString rulesuffix = wxEmptyString;
+   if ( ruleprefix.Find(':') >= 0 ) {
+      ruleprefix = ruleprefix.BeforeFirst(':');
+      rulesuffix = wxT(":") + rulestring.AfterFirst(':');
+   }
+
+   // first look for given rulestring in namedrules; if user has created a name like
+   // "Life on torus" for "B3/S23:T100,200" then this will find that name
+   for (size_t i = 0; i < namedrules.GetCount(); i++) {
+      wxString thisrule = namedrules[i].AfterFirst('|');       // rule is after '|'
+      if ( rulestring.IsSameAs(thisrule,false) ) {
+         rulename = namedrules[i].BeforeFirst('|');            // name is before '|'
+         if ( rulesuffix.Length() > 0 ) {
+            // still append suffix so user sees "Life on torus:T100,200"
+            rulename += rulesuffix;
+         }
          return rulename;
       }
    }
+
+   if ( rulesuffix.Length() > 0 ) {
+      // look for ruleprefix in namedrules; if there is no explicit name for
+      // "B3/S23:T100,200" then this will find "Life" and user will see "Life:T100,200"
+      for (size_t i = 0; i < namedrules.GetCount(); i++) {
+         wxString thisrule = namedrules[i].AfterFirst('|');    // rule is after '|'
+         if ( ruleprefix.IsSameAs(thisrule,false) ) {
+            rulename = namedrules[i].BeforeFirst('|');         // name is before '|'
+            rulename += rulesuffix;
+            return rulename;
+         }
+      }
+   }
+   
    // given rulestring has not been named
    rulename = rulestring;
    return rulename;
@@ -290,10 +293,16 @@ void AlgoHelp::OnMouseDown(wxMouseEvent& event)
 void AlgoHelp::DisplayFile(const wxString& filepath)
 {
    if ( filepath.IsEmpty() ) {
+      wxString newrule = ruletext->GetValue();
       wxString contents =
       wxT("<html><body bgcolor=\"#FFFFCE\">")
-      wxT("<p>The given rule is not valid in any algorithm.")
-      wxT("</body></html>");
+      wxT("<p>The given rule is not valid in any algorithm.");
+      if (newrule.Contains(wxT(":"))) {
+         // assume user is trying to specify a bounded grid
+         contents += wxT("<p>The syntax for a bounded grid is described ");
+         contents += wxT("<a href=\"Help/bounded.html\">here</a>.");
+      }
+      contents += wxT("</body></html>");
       SetPage(contents);
 
    } else if ( filepath.StartsWith(gollydir) &&
@@ -721,12 +730,11 @@ void RuleDialog::UpdateName()
       newindex = 0;
    } else {
       // search namedrules array for matching rule
-      size_t i;
       newindex = -1;
-      for (i=0; i<namedrules.GetCount(); i++) {
+      for (size_t i=0; i<namedrules.GetCount(); i++) {
          // extract rule after '|'
          wxString thisrule = namedrules[i].AfterFirst('|');
-         if ( MatchingRules(newrule, thisrule) ) {
+         if ( newrule.IsSameAs(thisrule,false) ) {
             newindex = i;
             break;
          }
@@ -760,6 +768,15 @@ void RuleDialog::OnRuleTextChanged(wxCommandEvent& WXUNUSED(event))
    if (ignore_text_change) return;
    UpdateName();
    UpdateAlgo();
+   
+   // check for spaces
+   wxString newrule = ruletext->GetValue();
+   if (newrule.Contains(wxT(" "))) {
+      Warning(_("Spaces are not allowed in rule strings."));
+      newrule.Replace(wxT(" "), wxEmptyString);
+      ruletext->SetValue(newrule);
+      ruletext->SetFocus();
+   }
 }
 
 // -----------------------------------------------------------------------------
@@ -779,8 +796,19 @@ void RuleDialog::OnChooseAlgo(wxCommandEvent& event)
       const char* err = tempalgo->setrule( thisrule.mb_str(wxConvLocal) );
       if (err) {
          // rule is not valid so change rule text to selected algo's default rule
+         wxString defrule = wxString(tempalgo->DefaultRule(), wxConvLocal);
+         if (oldindex < NumAlgos() && thisrule.Find(':') >= 0) {
+            // preserve valid topology so we can do things like switch from
+            // "LifeHistory:T30,20" in RuleTable to "B3/S23:T30,20" in QuickLife
+            if (defrule.Find(':') >= 0) {
+               // default rule shouldn't have a suffix but play safe and remove it
+               defrule = defrule.BeforeFirst(':');
+            }
+            defrule += wxT(":");
+            defrule += thisrule.AfterFirst(':');
+         }
          ignore_text_change = true;
-         ruletext->SetValue( wxString(tempalgo->DefaultRule(),wxConvLocal) );
+         ruletext->SetValue(defrule);
          ruletext->SetFocus();
          ruletext->SetSelection(-1,-1);
          ignore_text_change = false;
@@ -838,7 +866,7 @@ void RuleDialog::OnAddName(wxCommandEvent& WXUNUSED(event))
 {
    if ( nameindex < (int) namedrules.GetCount() ) {
       // OnUpdateAdd should prevent this but play safe
-      wxBell();
+      Beep();
       return;
    }
    
@@ -894,7 +922,7 @@ void RuleDialog::OnDeleteName(wxCommandEvent& WXUNUSED(event))
 {
    if ( nameindex <= 0 || nameindex >= (int) namedrules.GetCount() ) {
       // OnUpdateDelete should prevent this but play safe
-      wxBell();
+      Beep();
       return;
    }
    
