@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -37,11 +37,12 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include "wxview.h"        // for viewptr->...
 #include "wxrender.h"      // for SetSelectionColor
 #include "wxstatus.h"      // for statusptr->...
-#include "wxutils.h"       // for Warning, IsScriptFile
+#include "wxutils.h"       // for Warning, Beep, IsScriptFile
 #include "wxprefs.h"       // for gollydir, allowundo, etc
 #include "wxundo.h"        // for undoredo->...
 #include "wxalgos.h"       // for *_ALGO, algoinfo
 #include "wxlayer.h"       // for currlayer, SyncClones
+#include "wxtimeline.h"    // for TimelineExists
 #include "wxperl.h"        // for RunPerlScript, AbortPerlScript
 #include "wxpython.h"      // for RunPythonScript, AbortPythonScript
 #include "wxscript.h"
@@ -252,6 +253,10 @@ const char* GSF_setrule(char* rulestring)
    wxString oldrule = wxString(currlayer->algo->getrule(),wxConvLocal);
    int oldmaxstate = currlayer->algo->NumCellStates() - 1;
 
+   // selection might change if grid becomes smaller,
+   // so save current selection for RememberRuleChange/RememberAlgoChange
+   viewptr->SaveCurrentSelection();
+
    // inscript should be true but play safe
    if (allowundo && !currlayer->stayclean && inscript) {
       // note that we must save pending gen changes BEFORE changing rule
@@ -296,23 +301,29 @@ const char* GSF_setrule(char* rulestring)
       // show new rule in main window's title but don't change name
       ChangeWindowTitle(wxEmptyString);
 
+      // if grid is bounded then remove any live cells outside grid edges
+      if (currlayer->algo->gridwd > 0 || currlayer->algo->gridht > 0) {
+         mainptr->ClearOutsideGrid();
+      }
+
       // rule change might have changed the number of cell states;
       // if there are fewer states then pattern might change
       int newmaxstate = currlayer->algo->NumCellStates() - 1;
       if (newmaxstate < oldmaxstate && !currlayer->algo->isEmpty()) {
          mainptr->ReduceCellStates(newmaxstate);
       }
-
-      // switch to default colors for new rule
-      UpdateLayerColors();
-
-      // pattern might have changed or colors might have changed
-      DoAutoUpdate();
       
       if (allowundo && !currlayer->stayclean) {
          currlayer->undoredo->RememberRuleChange(oldrule);
       }
    }
+
+   // switch to default colors and icons for new rule (we need to do this even if
+   // oldrule == newrule in case there's a new/changed .colors or .icons file)
+   UpdateLayerColors();
+
+   // pattern or colors or icons might have changed
+   DoAutoUpdate();
    
    return NULL;
 }
@@ -345,6 +356,15 @@ const char* GSF_setpos(char* x, char* y)
 
    bigint bigx(x);
    bigint bigy(y);
+   
+   // check if x,y is outside bounded grid
+   if ( (currlayer->algo->gridwd > 0 &&
+            (bigx < currlayer->algo->gridleft || bigx > currlayer->algo->gridright)) ||
+        (currlayer->algo->gridht > 0 &&
+            (bigy < currlayer->algo->gridtop || bigy > currlayer->algo->gridbottom)) ) {
+      return "Given position is outside grid boundary.";
+   }
+   
    viewptr->SetPosMag(bigx, bigy, viewptr->GetMag());
    DoAutoUpdate();
 
@@ -400,10 +420,20 @@ void GSF_setname(char* name, int index)
 
 const char* GSF_setcell(int x, int y, int newstate)
 {
+   // check if x,y is outside bounded grid
+   if ( (currlayer->algo->gridwd > 0 &&
+            (x < currlayer->algo->gridleft.toint() ||
+             x > currlayer->algo->gridright.toint())) ||
+        (currlayer->algo->gridht > 0 &&
+            (y < currlayer->algo->gridtop.toint() ||
+             y > currlayer->algo->gridbottom.toint())) ) {
+      return "Given cell is outside grid boundary.";
+   }
+
    int oldstate = currlayer->algo->getcell(x, y);
    if (newstate != oldstate) {
       if (currlayer->algo->setcell(x, y, newstate) < 0) {
-         return "setcell error: state value is out of range.";
+         return "State value is out of range.";
       }
       currlayer->algo->endofpattern();
       if (allowundo && !currlayer->stayclean) {
@@ -419,8 +449,18 @@ const char* GSF_setcell(int x, int y, int newstate)
 
 const char* GSF_paste(int x, int y, char* mode)
 {
+   // check if x,y is outside bounded grid
+   if ( (currlayer->algo->gridwd > 0 &&
+            (x < currlayer->algo->gridleft.toint() ||
+             x > currlayer->algo->gridright.toint())) ||
+        (currlayer->algo->gridht > 0 &&
+            (y < currlayer->algo->gridtop.toint() ||
+             y > currlayer->algo->gridbottom.toint())) ) {
+      return "Given cell is outside grid boundary.";
+   }
+
    if (!mainptr->ClipboardHasText())
-      return "paste error: no pattern in clipboard.";
+      return "No pattern in clipboard.";
 
    // temporarily change selection and paste mode
    Selection oldsel = currlayer->currsel;
@@ -431,7 +471,7 @@ const char* GSF_paste(int x, int y, char* mode)
    else if (modestr.IsSameAs(wxT("copy"), false)) SetPasteMode("Copy");
    else if (modestr.IsSameAs(wxT("or"), false))   SetPasteMode("Or");
    else if (modestr.IsSameAs(wxT("xor"), false))  SetPasteMode("Xor");
-   else return "paste error: unknown mode.";
+   else return "Unknown mode.";
 
    // create huge selection rect so no possibility of error message
    currlayer->currsel.SetRect(x, y, INT_MAX, INT_MAX);
@@ -448,6 +488,97 @@ const char* GSF_paste(int x, int y, char* mode)
 
 // -----------------------------------------------------------------------------
 
+const char* GSF_checkpos(lifealgo* algo, int x, int y)
+{
+   // check that x,y is within bounded grid
+   if ( (algo->gridwd > 0 &&
+            (x < algo->gridleft.toint() || x > algo->gridright.toint())) ||
+        (algo->gridht > 0 &&
+            (y < algo->gridtop.toint() || y > algo->gridbottom.toint())) ) {
+      return "Cell is outside grid boundary.";
+   }
+   return NULL;
+}
+
+// -----------------------------------------------------------------------------
+
+const char* GSF_checkrect(int x, int y, int wd, int ht)
+{
+   if (wd <= 0) return "Rectangle width must be > 0.";
+   if (ht <= 0) return "Rectangle height must be > 0.";
+
+   // check that rect is completely within bounded grid
+   if ( (currlayer->algo->gridwd > 0 &&
+            (x < currlayer->algo->gridleft.toint() ||
+             x > currlayer->algo->gridright.toint() ||
+             x+wd-1 < currlayer->algo->gridleft.toint() ||
+             x+wd-1 > currlayer->algo->gridright.toint())) ||
+        (currlayer->algo->gridht > 0 &&
+            (y < currlayer->algo->gridtop.toint() ||
+             y > currlayer->algo->gridbottom.toint() ||
+             y+ht-1 < currlayer->algo->gridtop.toint() ||
+             y+ht-1 > currlayer->algo->gridbottom.toint())) ) {
+      return "Rectangle is outside grid boundary.";
+   }
+   return NULL;
+}
+
+// -----------------------------------------------------------------------------
+
+int GSF_hash(int x, int y, int wd, int ht)
+{
+   // calculate a hash value for pattern in given rect
+   int hash = 31415962;
+   int right = x + wd - 1;
+   int bottom = y + ht - 1;
+   int cx, cy;
+   int v = 0;
+
+   lifealgo* curralgo = currlayer->algo;
+   if (curralgo->NumCellStates() > 2) {
+      // multi-state universe
+      for ( cy=y; cy<=bottom; cy++ ) {
+         int yshift = cy - y;
+         for ( cx=x; cx<=right; cx++ ) {
+            int skip = curralgo->nextcell(cx, cy, v);
+            if (skip >= 0) {
+               // found next live cell in this row (v is >= 1)
+               cx += skip;
+               if (cx <= right) {
+                  // need to use a good hash function for patterns like AlienCounter.rle
+                  hash = (hash * 1000003) ^ yshift;
+                  hash = (hash * 1000003) ^ (cx - x);
+                  hash = (hash * 1000003) ^ v;
+               }
+            } else {
+               cx = right;  // done this row
+            }
+         }
+      }
+   } else {
+      // two-state universe
+      for ( cy=y; cy<=bottom; cy++ ) {
+         int yshift = cy - y;
+         for ( cx=x; cx<=right; cx++ ) {
+            int skip = curralgo->nextcell(cx, cy, v);
+            if (skip >= 0) {
+               // found next live cell in this row (v is 1)
+               cx += skip;
+               if (cx <= right) {
+                  hash = (hash * 33 + yshift) ^ (cx - x);
+               }
+            } else {
+               cx = right;  // done this row
+            }
+         }
+      }
+   }
+
+   return hash;
+}
+
+// -----------------------------------------------------------------------------
+
 void GSF_select(int x, int y, int wd, int ht)
 {
    if (wd < 1 || ht < 1) {
@@ -778,6 +909,14 @@ bool GSF_setcolor(char* colname, wxColor& newcol, wxColor& oldcol)
          DoAutoUpdate();
       }
 
+   } else if (strcmp(colname, "border") == 0) {
+      oldcol = *borderrgb;
+      if (oldcol != newcol) {
+         *borderrgb = newcol;
+         SetBrushesAndPens();
+         DoAutoUpdate();
+      }
+
    } else if (strcmp(colname, "paste") == 0) {
       oldcol = *pastergb;
       if (oldcol != newcol) {
@@ -842,6 +981,7 @@ bool GSF_getcolor(char* colname, wxColor& color)
       // deprecated; can now use getcolors(0)
       color.Set(currlayer->cellr[0], currlayer->cellg[0], currlayer->cellb[0]);
    }
+   else if (strcmp(colname, "border") == 0)     color = *borderrgb;
    else if (strcmp(colname, "paste") == 0)      color = *pastergb;
    else if (strcmp(colname, "select") == 0)     color = *selectrgb;
    // next two are deprecated
@@ -879,7 +1019,9 @@ void GSF_getkey(char* s)
 // -----------------------------------------------------------------------------
 
 // also allow mouse interaction??? ie. g.doclick( g.getclick() ) where
-// getclick returns [] or [x,y,button,modifiers]
+// getclick returns [] or [x,y,button,modifiers];
+// or maybe it would be better to implement Perl/Python callback routines
+// like OnKey and OnClick that a script can override???
 
 void GSF_dokey(char* ascii)
 {
@@ -971,7 +1113,7 @@ void CheckScriptError(const wxString& ext)
          errtype = _("Python error:");
          scripterr.Replace(wxT("  File \"<string>\", line 1, in ?\n"), wxT(""));
       }
-      wxBell();
+      Beep();
       #ifdef __WXMAC__
          wxSetCursor(*wxSTANDARD_CURSOR);
       #endif
@@ -1027,13 +1169,18 @@ void SavePendingChanges(bool checkgenchanges)
 
 void RunScript(const wxString& filename)
 {
+   if (TimelineExists()) {
+      statusptr->ErrorMessage(_("You can't run a script if there is a timeline."));
+      return;
+   }
+   
    // use these flags to allow re-entrancy
    bool already_inscript = inscript;
    bool in_plscript = plscript;
    bool in_pyscript = pyscript;
    wxString savecwd;
 
-   if ( !wxFileName::FileExists(filename) ) {
+   if (!wxFileName::FileExists(filename)) {
       Warning(_("The script file does not exist:\n") + filename);
       return;
    }
@@ -1090,11 +1237,11 @@ void RunScript(const wxString& filename)
       inscript = true;
    
       mainptr->UpdateUserInterface(mainptr->IsActive());
-      #ifdef __WXMSW__
-         // temporarily clear non-ctrl and non-func key accelerators from
-         // menu items so keys like tab/enter/space can be passed to script
-         mainptr->UpdateMenuAccelerators();
-      #endif
+
+      // temporarily clear non-ctrl and non-func key accelerators from
+      // menu items so keys like tab/enter/space can be passed to script
+      // even if menu item is disabled
+      mainptr->UpdateMenuAccelerators();
    }
 
    wxString ext = filename.AfterLast('.');
@@ -1188,10 +1335,8 @@ void RunScript(const wxString& filename)
       if (showtitle) mainptr->SetWindowTitle(wxEmptyString);
       mainptr->UpdateEverything();
       
-      #ifdef __WXMSW__
-         // restore accelerators that were cleared above
-         mainptr->UpdateMenuAccelerators();
-      #endif
+      // restore accelerators that were cleared above
+      mainptr->UpdateMenuAccelerators();
    }
 }
 
@@ -1200,6 +1345,10 @@ void RunScript(const wxString& filename)
 void PassKeyToScript(int key)
 {
    if (key == WXK_ESCAPE) {
+      if (mainptr->generating) {
+         // interrupt a run() or step() command
+         wxGetApp().PollerInterrupt();
+      }
       if (plscript) AbortPerlScript();
       if (pyscript) AbortPythonScript();
    } else {
@@ -1245,6 +1394,10 @@ void FinishScripting()
 {
    // called when main window is closing (ie. app is quitting)
    if (inscript) {
+      if (mainptr->generating) {
+         // interrupt a run() or step() command
+         wxGetApp().PollerInterrupt();
+      }
       if (plscript) AbortPerlScript();
       if (pyscript) AbortPythonScript();
       wxSetWorkingDirectory(gollydir);
