@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -168,6 +168,32 @@ void Selection::SetEdges(bigint& t, bigint& l, bigint& b, bigint& r)
    selleft = l;
    selbottom = b;
    selright = r;
+   CheckGridEdges();
+}
+
+// -----------------------------------------------------------------------------
+
+void Selection::CheckGridEdges()
+{
+   if (exists) {
+      // change selection edges if necessary to ensure they are inside a bounded grid
+      if (currlayer->algo->gridwd > 0) {
+         if (selleft > currlayer->algo->gridright || selright < currlayer->algo->gridleft) {
+            exists = false;   // selection is outside grid
+            return;
+         }
+         if (selleft < currlayer->algo->gridleft) selleft = currlayer->algo->gridleft;
+         if (selright > currlayer->algo->gridright) selright = currlayer->algo->gridright;
+      }
+      if (currlayer->algo->gridht > 0) {
+         if (seltop > currlayer->algo->gridbottom || selbottom < currlayer->algo->gridtop) {
+            exists = false;   // selection is outside grid
+            return;
+         }
+         if (seltop < currlayer->algo->gridtop) seltop = currlayer->algo->gridtop;
+         if (selbottom > currlayer->algo->gridbottom) selbottom = currlayer->algo->gridbottom;
+      }
+   }
 }
 
 // -----------------------------------------------------------------------------
@@ -256,6 +282,8 @@ void Selection::Advance()
    bool savecells = allowundo && !currlayer->stayclean;
    if (savecells && inscript) SavePendingChanges();
 
+   bool boundedgrid = (currlayer->algo->gridwd > 0 || currlayer->algo->gridht > 0);
+
    // check if selection encloses entire pattern;
    // can't do this if qlife because it uses gen parity to decide which bits to draw;
    // also avoid this if undo/redo is enabled (too messy to remember cell changes)
@@ -267,7 +295,9 @@ void Selection::Advance()
       bigint savegen = currlayer->algo->getGeneration();
       bigint saveinc = currlayer->algo->getIncrement();
       currlayer->algo->setIncrement(1);
+      if (boundedgrid) mainptr->CreateBorderCells(currlayer->algo);
       currlayer->algo->step();
+      if (boundedgrid) mainptr->DeleteBorderCells(currlayer->algo);
       currlayer->algo->setIncrement(saveinc);
       currlayer->algo->setGeneration(savegen);
    
@@ -313,7 +343,9 @@ void Selection::Advance()
    mainptr->generating = true;
    wxGetApp().PollerReset();
    tempalgo->setIncrement(1);
+   if (boundedgrid) mainptr->CreateBorderCells(tempalgo);
    tempalgo->step();
+   if (boundedgrid) mainptr->DeleteBorderCells(tempalgo);
    mainptr->generating = false;
    
    if ( !tempalgo->isEmpty() ) {
@@ -387,6 +419,8 @@ void Selection::AdvanceOutside()
    bool savecells = allowundo && !currlayer->stayclean;
    if (savecells && inscript) SavePendingChanges();
 
+   bool boundedgrid = (currlayer->algo->gridwd > 0 || currlayer->algo->gridht > 0);
+
    // check if selection is completely outside pattern edges;
    // can't do this if qlife because it uses gen parity to decide which bits to draw;
    // also avoid this if undo/redo is enabled (too messy to remember cell changes)
@@ -398,13 +432,15 @@ void Selection::AdvanceOutside()
       bigint savegen = currlayer->algo->getGeneration();
       bigint saveinc = currlayer->algo->getIncrement();
       currlayer->algo->setIncrement(1);
+      if (boundedgrid) mainptr->CreateBorderCells(currlayer->algo);
       currlayer->algo->step();
+      if (boundedgrid) mainptr->DeleteBorderCells(currlayer->algo);
       currlayer->algo->setIncrement(saveinc);
       currlayer->algo->setGeneration(savegen);
    
       mainptr->generating = false;
       
-      // if pattern expanded then may need to clear ONE edge of selection
+      // clear selection in case pattern expanded into it
       Clear();
       MarkLayerDirty();
       mainptr->UpdateEverything();
@@ -461,7 +497,9 @@ void Selection::AdvanceOutside()
    mainptr->generating = true;
    wxGetApp().PollerReset();
    currlayer->algo->setIncrement(1);
+   if (boundedgrid) mainptr->CreateBorderCells(currlayer->algo);
    currlayer->algo->step();
+   if (boundedgrid) mainptr->DeleteBorderCells(currlayer->algo);
    mainptr->generating = false;
    
    if ( !currlayer->algo->isEmpty() ) {
@@ -536,11 +574,24 @@ void Selection::AdvanceOutside()
    
    if (savecells) {
       // compare patterns in oldalgo and currlayer->algo and call SaveCellChange
-      // for each cell that has a different state; note that we expand the
-      // original pattern rect by 1 in case generating caused expansion
-      if ( SaveDifferences(oldalgo, currlayer->algo,
-                           top.toint() - 1, left.toint() - 1,
-                           bottom.toint() + 1, right.toint() + 1) ) {
+      // for each cell that has a different state; note that we need to compare
+      // the union of the original pattern's rect and the new pattern's rect
+      int otop = top.toint();
+      int oleft = left.toint();
+      int obottom = bottom.toint();
+      int oright = right.toint();
+      if (!currlayer->algo->isEmpty()) {
+         currlayer->algo->findedges(&top, &left, &bottom, &right);
+         int ntop = top.toint();
+         int nleft = left.toint();
+         int nbottom = bottom.toint();
+         int nright = right.toint();
+         if (ntop < otop) otop = ntop;
+         if (nleft < oleft) oleft = nleft;
+         if (nbottom > obottom) obottom = nbottom;
+         if (nright > oright) oright = nright;
+      }
+      if ( SaveDifferences(oldalgo, currlayer->algo, otop, oleft, obottom, oright) ) {
          delete oldalgo;
          if ( !currlayer->undoredo->RememberCellChanges(_("Advance Outside"), currlayer->dirty) ) {
             // pattern outside selection didn't change
@@ -862,8 +913,12 @@ bool Selection::Visible(wxRect* visrect)
    // all or some of selection is visible in viewport;
    // only set visible rectangle if requested
    if (visrect) {
+      // first we must clip coords to viewport
       if (lt.first < 0) lt.first = 0;
       if (lt.second < 0) lt.second = 0;
+      if (rb.first > currlayer->view->getxmax()) rb.first = currlayer->view->getxmax();
+      if (rb.second > currlayer->view->getymax()) rb.second = currlayer->view->getymax();
+      
       if (currlayer->view->getmag() > 0) {
          // move rb to pixel at bottom right corner of cell
          rb.first += (1 << currlayer->view->getmag()) - 1;
@@ -873,9 +928,11 @@ bool Selection::Visible(wxRect* visrect)
             rb.first--;
             rb.second--;
          }
+         // clip to viewport again
+         if (rb.first > currlayer->view->getxmax()) rb.first = currlayer->view->getxmax();
+         if (rb.second > currlayer->view->getymax()) rb.second = currlayer->view->getymax();
       }
-      if (rb.first > currlayer->view->getxmax()) rb.first = currlayer->view->getxmax();
-      if (rb.second > currlayer->view->getymax()) rb.second = currlayer->view->getymax();
+
       visrect->SetX(lt.first);
       visrect->SetY(lt.second);
       visrect->SetWidth(rb.first - lt.first + 1);
@@ -1895,6 +1952,15 @@ bool Selection::Rotate(bool clockwise, bool inundoredo)
    bigint newleft   = midx;   newleft   += seltop;      newleft   -= midy;
    bigint newright  = midx;   newright  += selbottom;   newright  -= midy;
    
+   // check if rotated selection edges are outside bounded grid
+   if ( (currlayer->algo->gridwd > 0 &&
+            (newleft < currlayer->algo->gridleft || newright > currlayer->algo->gridright)) ||
+        (currlayer->algo->gridht > 0 &&
+            (newtop < currlayer->algo->gridtop || newbottom > currlayer->algo->gridbottom)) ) {
+      statusptr->ErrorMessage(_("New selection would be outside grid boundary."));
+      return false;
+   }
+   
    // if there is no pattern then just rotate the selection edges
    if (currlayer->algo->isEmpty()) {
       viewptr->SaveCurrentSelection();
