@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -33,13 +33,14 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include "lifealgo.h"
 
 #include "wxgolly.h"       // for wxGetApp, etc
-#include "wxutils.h"       // for Fatal, FillRect
+#include "wxutils.h"       // for Fatal, Beep, FillRect
 #include "wxprefs.h"       // for mindelay, maxdelay, etc
 #include "wxview.h"        // for viewptr->...
 #include "wxmain.h"        // for mainptr->...
 #include "wxscript.h"      // for inscript
 #include "wxalgos.h"       // for algoinfo
 #include "wxlayer.h"       // for currlayer
+#include "wxtimeline.h"    // for TimelineExists
 #include "wxstatus.h"
 
 // -----------------------------------------------------------------------------
@@ -118,7 +119,7 @@ void StatusBar::DisplayMessage(const wxString& s)
 void StatusBar::ErrorMessage(const wxString& s)
 {
    if (inscript) return;                     // let script control messages
-   wxBell();
+   Beep();
    DisplayMessage(s);
 }
 
@@ -189,7 +190,7 @@ void StatusBar::SetStatusFont(wxDC& dc)
 {
    dc.SetFont(*statusfont);
    dc.SetTextForeground(*wxBLACK);
-   dc.SetBrush(*wxBLACK_BRUSH);           // avoids problem on Linux/X11
+   dc.SetBrush(*wxBLACK_BRUSH);
    dc.SetBackgroundMode(wxTRANSPARENT);
 }
 
@@ -432,7 +433,7 @@ void StatusBar::OnPaint(wxPaintEvent& WXUNUSED(event))
       // use wxWidgets buffering to avoid flicker
       int wd, ht;
       GetClientSize(&wd, &ht);
-      // wd or ht might be < 1 on Win/X11 platforms
+      // wd or ht might be < 1 on Windows
       if (wd < 1) wd = 1;
       if (ht < 1) ht = 1;
       if (wd != statbitmapwd || ht != statbitmapht) {
@@ -488,7 +489,11 @@ void StatusBar::OnMouseDown(wxMouseEvent& event)
    ClearMessage();
 
    if ( ClickInGenBox(event.GetX(), event.GetY()) && !mainptr->generating ) {
-      mainptr->SetGeneration();
+      if (TimelineExists()) {
+         ErrorMessage(_("You can't change the generation count if there is a timeline."));
+      } else {
+         mainptr->SetGeneration();
+      }
 
    } else if ( ClickInScaleBox(event.GetX(), event.GetY()) ) {
       if (viewptr->GetMag() != 0) {
@@ -497,8 +502,10 @@ void StatusBar::OnMouseDown(wxMouseEvent& event)
       }
 
    } else if ( ClickInStepBox(event.GetX(), event.GetY()) ) {
-      if (currlayer->currbase != algoinfo[currlayer->algtype]->defbase ||
-          currlayer->currexpo != 0) {
+      if (TimelineExists()) {
+         ErrorMessage(_("You can't change the step size if there is a timeline."));
+      } else if (currlayer->currbase != algoinfo[currlayer->algtype]->defbase ||
+                 currlayer->currexpo != 0) {
          // reset base step to default value and step exponent to 0
          currlayer->currbase = algoinfo[currlayer->algtype]->defbase;
          mainptr->SetStepExponent(0);
@@ -507,11 +514,6 @@ void StatusBar::OnMouseDown(wxMouseEvent& event)
          Update();
       }
    }
-
-   #ifdef __WXX11__
-      // make sure viewport keeps keyboard focus
-      viewptr->SetFocus();
-   #endif
 }
 
 // -----------------------------------------------------------------------------
@@ -568,7 +570,7 @@ StatusBar::StatusBar(wxWindow* parent, wxCoord xorg, wxCoord yorg, int wd, int h
    h_scale = h_pop + textwd + mingap;
    dc.GetTextExtent(_("Scale=2^9999:1"), &textwd, &textht);
    h_step = h_scale + textwd + mingap;
-   dc.GetTextExtent(_("Step=10^9999"), &textwd, &textht);
+   dc.GetTextExtent(_("Step=1000000000^9"), &textwd, &textht);
    h_xy = h_step + textwd + mingap;
    // when showexact is true:
    dc.GetTextExtent(_("X = "), &textwd, &textht);
