@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -35,6 +35,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include "wxview.h"        // for viewptr->...
 #include "wxmain.h"        // for mainptr->...
 #include "wxscript.h"      // for inscript, PassKeyToScript
+#include "wxprefs.h"       // for allowbeep
 #include "wxutils.h"
 
 #ifdef __WXMAC__
@@ -68,7 +69,7 @@ void Note(const wxString& msg)
 
 void Warning(const wxString& msg)
 {
-   wxBell();
+   Beep();
    wxString title = wxGetApp().GetAppName() + _(" warning:");
    #ifdef __WXMAC__
       wxSetCursor(*wxSTANDARD_CURSOR);
@@ -80,14 +81,21 @@ void Warning(const wxString& msg)
 
 void Fatal(const wxString& msg)
 {
-   wxBell();
+   Beep();
    wxString title = wxGetApp().GetAppName() + _(" error:");
    #ifdef __WXMAC__
       wxSetCursor(*wxSTANDARD_CURSOR);
    #endif
    wxMessageBox(msg, title, wxOK | wxICON_ERROR, wxGetActiveWindow());
-   // calling wxExit() results in a bus error on X11
-   exit(1);
+
+   exit(1);    // safer than calling wxExit()
+}
+
+// -----------------------------------------------------------------------------
+
+void Beep()
+{
+   if (allowbeep) wxBell();
 }
 
 // =============================================================================
@@ -121,6 +129,8 @@ StringDialog::StringDialog(wxWindow* parent, const wxString& title,
    SetSizer(topSizer);
 
    textbox = new wxTextCtrl(this, wxID_ANY, instring);
+                            // add wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE
+                            // as an option to allow multi-line input???!!!
    wxStaticText* promptlabel = new wxStaticText(this, wxID_STATIC, prompt);
 
    wxSizer* stdbutts = CreateButtonSizer(wxOK | wxCANCEL);
@@ -247,7 +257,7 @@ void IntegerDialog::OnSpinCtrlChar(wxKeyEvent& event)
          event.Skip();
       } else {
          // disallow any other displayable ascii char
-         wxBell();
+         Beep();
       }
 
    } else {
@@ -440,11 +450,7 @@ int SaveChanges(const wxString& query, const wxString& msg)
 // globals for showing progress
 
 wxProgressDialog* progdlg = NULL;         // progress dialog
-#ifdef __WXX11__
-   const int maxprogrange = 10000;        // maximum range must be < 32K on X11?
-#else
-   const int maxprogrange = 1000000000;   // maximum range (best if very large)
-#endif
+const int maxprogrange = 1000000000;      // maximum range (best if very large)
 wxStopWatch* progwatch = NULL;            // stopwatch for progress dialog
 long prognext;                            // when to update progress dialog
 wxString progtitle;                       // title for progress dialog
@@ -510,11 +516,7 @@ bool AbortProgress(double fraction_done, const wxString& newmsg)
    long msecs = progwatch->Time();
    if (progdlg) {
       if (msecs < prognext) return false;
-      #ifdef __WXX11__
-         prognext = msecs + 1000;    // call Update/Pulse about once per sec on X11
-      #else
-         prognext = msecs + 100;     // call Update/Pulse about 10 times per sec
-      #endif
+      prognext = msecs + 100;    // call Update/Pulse about 10 times per sec
       bool cancel;
       if (fraction_done < 0.0) {
          // show indeterminate progress gauge
@@ -565,10 +567,6 @@ void EndProgress()
       #endif
       delete progdlg;
       progdlg = NULL;
-      #ifdef __WXX11__
-         // fix activate problem on X11 if user hit Cancel button
-         mainptr->SetFocus();
-      #endif
    }
    if (progwatch) {
       delete progwatch;
