@@ -1,7 +1,7 @@
                         /*** /
 
 This file is part of Golly, a Game of Life Simulator.
-Copyright (C) 2009 Andrew Trevorrow and Tomas Rokicki.
+Copyright (C) 2010 Andrew Trevorrow and Tomas Rokicki.
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -35,6 +35,9 @@ void Warning(const wxString& msg);
 void Fatal(const wxString& msg);
 // Beep, display message in a modal dialog, then exit app.
 
+void Beep();
+// Play beep sound, depending on preference setting.
+
 bool GetString(const wxString& title, const wxString& prompt,
                const wxString& instring, wxString& outstring);
 // Display a dialog box to get a string from the user.
